@@ -1,52 +1,27 @@
-/* $Id: xmesa.h,v 1.8 1997/09/29 23:22:16 brianp Exp $ */
+/* $Id: xmesa.h,v 1.8 2000/05/02 02:30:01 brianp Exp $ */
 
 /*
  * Mesa 3-D graphics library
- * Version:  2.5
- * Copyright (C) 1995-1997  Brian Paul
- *
- * This library is free software; you can redistribute it and/or
- * modify it under the terms of the GNU Library General Public
- * License as published by the Free Software Foundation; either
- * version 2 of the License, or (at your option) any later version.
- *
- * This library is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- * Library General Public License for more details.
- *
- * You should have received a copy of the GNU Library General Public
- * License along with this library; if not, write to the Free
- * Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
- */
-
-
-/*
- * $Log: xmesa.h,v $
- * Revision 1.8  1997/09/29 23:22:16  brianp
- * updated version to 2.5
- *
- * Revision 1.7  1997/07/11 23:06:55  brianp
- * added XMesaGetDepthBuffer function (Randy Frank)
- *
- * Revision 1.6  1997/05/16 01:54:28  brianp
- * edited a comment
- *
- * Revision 1.5  1997/05/14 03:28:37  brianp
- * added XMesaDitherColor()
- *
- * Revision 1.4  1997/04/27 03:21:34  brianp
- * added XMesaFindBuffer() and XMesaGarbageCollect
- *
- * Revision 1.3  1996/10/30 03:13:01  brianp
- * incremented version to 2.1
- *
- * Revision 1.2  1996/09/20 02:56:45  brianp
- * updated for new XMesaContext, XMesaVisual and XMesaBuffer datatypes
- *
- * Revision 1.1  1996/09/13 01:26:41  brianp
- * Initial revision
- *
+ * Version:  3.3
+ * 
+ * Copyright (C) 1999-2000  Brian Paul   All Rights Reserved.
+ * 
+ * Permission is hereby granted, free of charge, to any person obtaining a
+ * copy of this software and associated documentation files (the "Software"),
+ * to deal in the Software without restriction, including without limitation
+ * the rights to use, copy, modify, merge, publish, distribute, sublicense,
+ * and/or sell copies of the Software, and to permit persons to whom the
+ * Software is furnished to do so, subject to the following conditions:
+ * 
+ * The above copyright notice and this permission notice shall be included
+ * in all copies or substantial portions of the Software.
+ * 
+ * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
+ * OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
+ * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL
+ * BRIAN PAUL BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN
+ * AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
+ * CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
  */
 
 
@@ -98,8 +73,13 @@ extern "C" {
 #endif
 
 
+#ifdef XFree86Server
+#include "xmesa_xf86.h"
+#else
 #include <X11/Xlib.h>
 #include <X11/Xutil.h>
+#include "xmesa_x.h"
+#endif
 #include "GL/gl.h"
 
 #ifdef AMIWIN
@@ -108,8 +88,8 @@ extern struct Library *XLibBase;
 #endif
 
 
-#define XMESA_MAJOR_VERSION 2
-#define XMESA_MINOR_VERSION 5
+#define XMESA_MAJOR_VERSION 3
+#define XMESA_MINOR_VERSION 3
 
 
 
@@ -120,6 +100,13 @@ extern struct Library *XLibBase;
 #define XMESA_EXTENSIONS 2
 
 
+/*
+ * Values passed to XMesaSetFXmode:
+ */
+#define XMESA_FX_WINDOW       1
+#define XMESA_FX_FULLSCREEN   2
+
+
 
 typedef struct xmesa_context *XMesaContext;
 
@@ -129,7 +116,6 @@ typedef struct xmesa_buffer *XMesaBuffer;
 
 
 
-
 /*
  * Create a new X/Mesa visual.
  * Input:  display - X11 display
@@ -139,23 +125,36 @@ typedef struct xmesa_buffer *XMesaBuffer;
  *         alpha_flag - alpha buffer requested?
  *         db_flag - GL_TRUE = double-buffered,
  *                   GL_FALSE = single buffered
- *         depth_size - requested bits/depth values, or zero
- *         stencil_size - requested bits/stencil values, or zero
- *         accum_size - requested bits/component values, or zero
+ *         stereo_flag - stereo visual?
  *         ximage_flag - GL_TRUE = use an XImage for back buffer,
  *                       GL_FALSE = use an off-screen pixmap for back buffer
+ *         depth_size - requested bits/depth values, or zero
+ *         stencil_size - requested bits/stencil values, or zero
+ *         accum_red_size - requested bits/red accum values, or zero
+ *         accum_green_size - requested bits/green accum values, or zero
+ *         accum_blue_size - requested bits/blue accum values, or zero
+ *         accum_alpha_size - requested bits/alpha accum values, or zero
+ *         num_samples - number of samples/pixel if multisampling, or zero
+ *         level - visual level, usually 0
+ *         visualCaveat - ala the GLX extension, usually GLX_NONE_EXT
  * Return;  a new XMesaVisual or 0 if error.
  */
-extern XMesaVisual XMesaCreateVisual( Display *display,
-                                      XVisualInfo *visinfo,
+extern XMesaVisual XMesaCreateVisual( XMesaDisplay *display,
+                                      XMesaVisualInfo visinfo,
                                       GLboolean rgb_flag,
                                       GLboolean alpha_flag,
                                       GLboolean db_flag,
+                                      GLboolean stereo_flag,
                                       GLboolean ximage_flag,
                                       GLint depth_size,
                                       GLint stencil_size,
-                                      GLint accum_size,
-                                      GLint level );
+                                      GLint accum_red_size,
+                                      GLint accum_green_size,
+                                      GLint accum_blue_size,
+                                      GLint accum_alpha_size,
+                                      GLint num_samples,
+                                      GLint level,
+                                      GLint visualCaveat );
 
 /*
  * Destroy an XMesaVisual, but not the associated XVisualInfo.
@@ -172,8 +171,8 @@ extern void XMesaDestroyVisual( XMesaVisual v );
  *                      lists or NULL if no sharing is wanted.
  * Return:  an XMesaContext or NULL if error.
  */
-extern XMesaContext XMesaCreateContext( XMesaVisual visual,
-                                        XMesaContext share_list );
+extern XMesaContext XMesaCreateContext( XMesaVisual v,
+					XMesaContext share_list );
 
 
 /*
@@ -185,14 +184,15 @@ extern void XMesaDestroyContext( XMesaContext c );
 /*
  * Create an XMesaBuffer from an X window.
  */
-extern XMesaBuffer XMesaCreateWindowBuffer( XMesaVisual v, Window w );
+extern XMesaBuffer XMesaCreateWindowBuffer( XMesaVisual v, XMesaWindow w );
 
 
 /*
  * Create an XMesaBuffer from an X pixmap.
  */
-extern XMesaBuffer XMesaCreatePixmapBuffer( XMesaVisual v, Pixmap p,
-                                            Colormap c );
+extern XMesaBuffer XMesaCreatePixmapBuffer( XMesaVisual v,
+					    XMesaPixmap p,
+					    XMesaColormap cmap );
 
 
 /*
@@ -203,16 +203,35 @@ extern void XMesaDestroyBuffer( XMesaBuffer b );
 
 /*
  * Return the XMesaBuffer handle which corresponds to an X drawable, if any.
+ *
  * New in Mesa 2.3.
  */
-extern XMesaBuffer XMesaFindBuffer( Display *dpy, Drawable d );
+extern XMesaBuffer XMesaFindBuffer( XMesaDisplay *dpy,
+				    XMesaDrawable d );
 
 
 
 /*
  * Bind a buffer to a context and make the context the current one.
  */
-extern GLboolean XMesaMakeCurrent( XMesaContext c, XMesaBuffer b );
+extern GLboolean XMesaMakeCurrent( XMesaContext c,
+				   XMesaBuffer b );
+
+
+/*
+ * Bind two buffers (read and draw) to a context and make the
+ * context the current one.
+ * New in Mesa 3.3
+ */
+extern GLboolean XMesaMakeCurrent2( XMesaContext c,
+                                    XMesaBuffer drawBuffer,
+                                    XMesaBuffer readBuffer );
+
+
+/*
+ * Unbind the current context from its buffer.
+ */
+extern GLboolean XMesaUnbindContext( XMesaContext c );
 
 
 /*
@@ -222,11 +241,18 @@ extern XMesaContext XMesaGetCurrentContext( void );
 
 
 /*
- * Return handle to the current buffer.
+ * Return handle to the current (draw) buffer.
  */
 extern XMesaBuffer XMesaGetCurrentBuffer( void );
 
 
+/*
+ * Return handle to the current read buffer.
+ * New in Mesa 3.3
+ */
+extern XMesaBuffer XMesaGetCurrentReadBuffer( void );
+
+
 /*
  * Swap the front and back buffers for the given buffer.  No action is
  * taken if the buffer is not double buffered.
@@ -234,6 +260,18 @@ extern XMesaBuffer XMesaGetCurrentBuffer( void );
 extern void XMesaSwapBuffers( XMesaBuffer b );
 
 
+/*
+ * Copy a sub-region of the back buffer to the front buffer.
+ *
+ * New in Mesa 2.6
+ */
+extern void XMesaCopySubBuffer( XMesaBuffer b,
+				int x,
+				int y,
+				int width,
+				int height );
+
+
 /*
  * Return a pointer to the the Pixmap or XImage being used as the back
  * color buffer of an XMesaBuffer.  This function is a way to get "under
@@ -245,7 +283,8 @@ extern void XMesaSwapBuffers( XMesaBuffer b );
  *          GL_FALSE = context is single buffered
  */
 extern GLboolean XMesaGetBackBuffer( XMesaBuffer b,
-                                     Pixmap *pixmap, XImage **ximage );
+				     XMesaPixmap *pixmap,
+				     XMesaImage **ximage );
 
 
 
@@ -260,8 +299,10 @@ extern GLboolean XMesaGetBackBuffer( XMesaBuffer b,
  * New in Mesa 2.4.
  */
 extern GLboolean XMesaGetDepthBuffer( XMesaBuffer b,
-                                      GLint *width, GLint *height,
-                                      GLint *bytesPerValue, void **buffer );
+				      GLint *width,
+				      GLint *height,
+				      GLint *bytesPerValue,
+				      void **buffer );
 
 
 
@@ -299,9 +340,27 @@ extern void XMesaGarbageCollect( void );
  *
  * New in Mesa 2.3.
  */
-extern unsigned long XMesaDitherColor( XMesaContext c, GLint x, GLint y,
-                                       GLfloat red, GLfloat green,
-                                       GLfloat blue, GLfloat alpha );
+extern unsigned long XMesaDitherColor( XMesaContext xmesa,
+				       GLint x,
+				       GLint y,
+				       GLfloat red,
+				       GLfloat green,
+				       GLfloat blue,
+				       GLfloat alpha );
+
+
+
+/*
+ * 3Dfx Glide driver only!
+ * Set 3Dfx/Glide full-screen or window rendering mode.
+ * Input:  mode - either XMESA_FX_WINDOW (window rendering mode) or
+ *                XMESA_FX_FULLSCREEN (full-screen rendering mode)
+ * Return:  GL_TRUE if success
+ *          GL_FALSE if invalid mode or if not using 3Dfx driver
+ *
+ * New in Mesa 2.6.
+ */
+extern GLboolean XMesaSetFXmode( GLint mode );
 
 
 
