@@ -1,4 +1,4 @@
-/* $Id: wmesa.c,v 3.1 1998/06/11 01:42:08 brianp Exp $ */
+/* $Id: wmesa.c,v 1.3 2000/03/03 23:21:57 brianp Exp $ */
 
 /*
 *   File name   :   wmesa.c
@@ -22,6 +22,42 @@
 
 /*
  * $Log: wmesa.c,v $
+ * Revision 1.3  2000/03/03 23:21:57  brianp
+ * removed obsolete logicop function
+ *
+ * Revision 1.2  2000/02/17 20:52:02  brianp
+ * replaced renderer_string() with get_string() func
+ *
+ * Revision 1.1.1.1  1999/08/19 00:55:42  jtg
+ * Imported sources
+ *
+ * Revision 3.10  1999/06/15 01:35:06  brianp
+ * small change to wmSetPixel() from TWILMOT@cpr.fr
+ *
+ * Revision 3.9  1999/05/11 19:06:01  brianp
+ * fixed a few VB->Index bugs (mikec@ensoniq.com)
+ *
+ * Revision 3.8  1999/05/08 15:15:23  brianp
+ * various updates from mikec@ensoniq.com
+ *
+ * Revision 3.7  1999/04/01 01:27:34  brianp
+ * always flip Y coord in read_rgba_span()
+ *
+ * Revision 3.6  1999/03/28 21:17:27  brianp
+ * updated SetBuffer driver function
+ *
+ * Revision 3.5  1999/03/16 01:36:42  brianp
+ * patched dither() to check if Current is NULL, per xzhou@nyx.net
+ *
+ * Revision 3.4  1999/02/25 14:12:33  keithw
+ * Merged in kw3 patch
+ *
+ * Revision 3.3  1999/01/03 03:08:57  brianp
+ * Ted Jump's changes
+ *
+ * Revision 3.2  1998/08/29 00:26:01
+ * updated for Mesa 3.0 to accomodate EGCS-Mingw32 build
+ *
  * Revision 3.1  1998/06/11 01:42:08  brianp
  * updated for Mesa 3.0 device driver interface (but not tested)
  *
@@ -115,6 +151,9 @@ void Mesa_Assert(void *Cond,void *File,unsigned Line)
 //#define FLIP(Y)  (Current->dither_flag? Y : Current->height-(Y)-1)
 //#define FLIP(Y)  (Current->height-(Y)-1)
 //#define FLIP(Y) Y
+/*
+ * XXX Why only flip Y coord if single buffered???
+ */
 #define FLIP(Y)  (Current->db_flag? Y: Current->height-(Y)-1)
 #define STARTPROFILE
 #define ENDPROFILE(PARA)
@@ -538,20 +577,11 @@ static GLboolean color_mask( GLcontext* ctx,
 
 
 
-/*
-* Set the pixel logic operation.  Return GL_TRUE if the device driver
-* can perform the operation, otherwise return GL_FALSE.  If GL_FALSE
-* is returned, the logic op will be done in software by Mesa.
-*/
-GLboolean logicop( GLcontext* ctx, GLenum op )
-{
-    /* can't implement */
-    return GL_FALSE;
-}
-
-
 static void dither( GLcontext* ctx, GLboolean enable )
 {
+   if (!Current)
+      return;
+
     if(enable == GL_FALSE){
         Current->dither_flag = GL_FALSE;
         if(Current->cColorBits == 8)
@@ -573,7 +603,7 @@ static GLboolean set_buffer( GLcontext* ctx, GLenum mode )
 {
    STARTPROFILE
    /* TODO: this could be better */
-   if (mode==GL_FRONT || mode==GL_BACK) {
+   if (mode==GL_FRONT_LEFT || mode==GL_BACK_LEFT) {
       return GL_TRUE;
    }
    else {
@@ -642,13 +672,13 @@ static void fast_rgb_points( GLcontext* ctx, GLuint first, GLuint last )
 
     STARTPROFILE
 
-        if (Current->gl_ctx->VB->MonoColor) {
+        if (0 /*Current->gl_ctx->VB->MonoColor*/) {
             /* all drawn with current color */
             for (i=first;i<=last;i++) {
                 if (!Current->gl_ctx->VB->ClipMask[i]) {
                     int x, y;
-                    x =       (GLint) Current->gl_ctx->VB->Win[i][0];
-                    y = FLIP( (GLint) Current->gl_ctx->VB->Win[i][1] );
+                    x =       (GLint) Current->gl_ctx->VB->Win.data[i][0];
+                    y = FLIP( (GLint) Current->gl_ctx->VB->Win.data[i][1] );
                     wmSetPixel(pwc, y,x,GetRValue(Current->pixel),
                         GetGValue(Current->pixel), GetBValue(Current->pixel));
                 }
@@ -659,14 +689,14 @@ static void fast_rgb_points( GLcontext* ctx, GLuint first, GLuint last )
             for (i=first;i<=last;i++) {
                 if (!Current->gl_ctx->VB->ClipMask[i]) {
                     int x, y;
-                    unsigned long pixel=RGB(Current->gl_ctx->VB->Color[i][0]*255.0,
-                        Current->gl_ctx->VB->Color[i][1]*255.0,
-                        Current->gl_ctx->VB->Color[i][2]*255.0);
-                    x =       (GLint) Current->gl_ctx->VB->Win[i][0];
-                    y = FLIP( (GLint) Current->gl_ctx->VB->Win[i][1] );
-                    wmSetPixel(pwc, y,x,Current->gl_ctx->VB->Color[i][0]*255.0,
-                        Current->gl_ctx->VB->Color[i][1]*255.0,
-                        Current->gl_ctx->VB->Color[i][2]*255.0);
+                    unsigned long pixel=RGB(Current->gl_ctx->VB->ColorPtr->data[i][0]*255.0,
+                        Current->gl_ctx->VB->ColorPtr->data[i][1]*255.0,
+                        Current->gl_ctx->VB->ColorPtr->data[i][2]*255.0);
+                    x =       (GLint) Current->gl_ctx->VB->Win.data[i][0];
+                    y = FLIP( (GLint) Current->gl_ctx->VB->Win.data[i][1] );
+                    wmSetPixel(pwc, y,x,Current->gl_ctx->VB->ColorPtr->data[i][0]*255.0,
+                        Current->gl_ctx->VB->ColorPtr->data[i][1]*255.0,
+                        Current->gl_ctx->VB->ColorPtr->data[i][2]*255.0);
                 }
             }
         }
@@ -693,7 +723,7 @@ extern points_func choose_points_function( GLcontext* ctx )
 
 
 
-/* Draw a line using the color specified by Current->gl_ctx->VB->Color[pv] */
+/* Draw a line using the color specified by Current->gl_ctx->VB->ColorPtr->data[pv] */
 static void fast_flat_rgb_line( GLcontext* ctx, GLuint v0, GLuint v1, GLuint pv )
 {
     STARTPROFILE
@@ -703,17 +733,17 @@ static void fast_flat_rgb_line( GLcontext* ctx, GLuint v0, GLuint v1, GLuint pv
     HPEN Pen;
     HPEN Old_Pen;
 
-    if (Current->gl_ctx->VB->MonoColor) {
+    if (0 /*Current->gl_ctx->VB->MonoColor*/) {
         pixel = Current->pixel;  /* use current color */
     }
     else {
-        pixel = RGB(Current->gl_ctx->VB->Color[pv][0]*255.0, Current->gl_ctx->VB->Color[pv][1]*255.0, Current->gl_ctx->VB->Color[pv][2]*255.0);
+        pixel = RGB(Current->gl_ctx->VB->ColorPtr->data[pv][0]*255.0, Current->gl_ctx->VB->ColorPtr->data[pv][1]*255.0, Current->gl_ctx->VB->ColorPtr->data[pv][2]*255.0);
     }
 
-    x0 =       (int) Current->gl_ctx->VB->Win[v0][0];
-    y0 = FLIP( (int) Current->gl_ctx->VB->Win[v0][1] );
-    x1 =       (int) Current->gl_ctx->VB->Win[v1][0];
-    y1 = FLIP( (int) Current->gl_ctx->VB->Win[v1][1] );
+    x0 =       (int) Current->gl_ctx->VB->Win.data[v0][0];
+    y0 = FLIP( (int) Current->gl_ctx->VB->Win.data[v0][1] );
+    x1 =       (int) Current->gl_ctx->VB->Win.data[v1][0];
+    y1 = FLIP( (int) Current->gl_ctx->VB->Win.data[v1][1] );
 
 
     BEGINGDICALL
@@ -1068,7 +1098,8 @@ static void read_rgba_span( const GLcontext* ctx,
    COLORREF Color;
    HDC DC=DD_GETDC;
    assert(Current->rgb_flag==GL_TRUE);
-   y=FLIP(y);
+   /*   y=FLIP(y);*/
+   y = Current->height - y - 1;
    for (i=0; i<n; i++) {
       Color=GetPixel(DC,x+i,y);
       rgba[i][RCOMP] = GetRValue(Color);
@@ -1114,16 +1145,21 @@ static void read_rgba_pixels( const GLcontext* ctx,
 /**********************************************************************/
 
 
-static const char *renderer_string(void)
+static const GLubyte *get_string(GLcontext *ctx, GLenum name)
 {
-   return "Windows";
+   if (name == GL_RENDERER) {
+      return (GLubyte *) "Mesa Windows";
+   }
+   else {
+      return NULL;
+   }
 }
 
 
 
 void setup_DD_pointers( GLcontext* ctx )
 {
-    ctx->Driver.RendererString = renderer_string;
+    ctx->Driver.GetString = get_string;
     ctx->Driver.UpdateState = setup_DD_pointers;
     ctx->Driver.GetBufferSize = buffer_size;
     ctx->Driver.Finish = finish;
@@ -1138,7 +1174,6 @@ void setup_DD_pointers( GLcontext* ctx )
     ctx->Driver.IndexMask = index_mask;
     ctx->Driver.ColorMask = color_mask;
 
-    ctx->Driver.LogicOp = logicop;
     ctx->Driver.Dither = dither;
 
     ctx->Driver.SetBuffer = set_buffer;
@@ -1527,11 +1562,11 @@ void /*WINAPI*/ wmCreatePalette( PWMC pwdc )
 
 void /*WINAPI*/ wmSetPixel(PWMC pwc, int iScanLine, int iPixel, BYTE r, BYTE g, BYTE b)
 {
-    if(Current->db_flag){
+    if (Current->db_flag) {
         LPBYTE  lpb = pwc->pbPixels;
         LPDWORD lpdw;
         LPWORD  lpw;
-        UINT    nBypp = pwc->cColorBits / 8;
+        UINT    nBypp = pwc->cColorBits >> 3;
         UINT    nOffset = iPixel % nBypp;
 
         // Move the pixel buffer pointer to the scanline that we
@@ -1561,8 +1596,7 @@ void /*WINAPI*/ wmSetPixel(PWMC pwc, int iScanLine, int iPixel, BYTE r, BYTE g,
             *lpdw = BGR32(r,g,b);
     }
     else{
-        HDC DC = DD_GETDC;
-        SetPixel(DC, iPixel, iScanLine, RGB(r,g,b));
+        SetPixel(Current->hDC, iPixel, iScanLine, RGB(r,g,b));
         DD_RELEASEDC;
     }
 }
@@ -2138,10 +2172,16 @@ static void smooth_8A8B8G8R_z_triangle( GLcontext *ctx,
     ffz += fdzdx;                           \
     }                                   \
     }
-#ifdef WIN32
-	#include "..\tritemp.h"
-#else
+
+#ifdef __MINGW32__
 	#include "tritemp.h"
+#else
+
+	#ifdef WIN32
+		#include "..\tritemp.h"
+	#else
+		#include "tritemp.h"
+	#endif
 #endif
 }
 
@@ -2174,10 +2214,15 @@ static void smooth_8R8G8B_z_triangle( GLcontext *ctx,
     ffz += fdzdx;                           \
     }                                   \
     }
-#ifdef WIN32
-	#include "..\tritemp.h"
-#else
+#ifdef __MINGW32__
 	#include "tritemp.h"
+#else
+
+	#ifdef WIN32
+		#include "..\tritemp.h"
+	#else
+		#include "tritemp.h"
+	#endif
 #endif
 }
 
@@ -2211,10 +2256,15 @@ static void smooth_5R6G5B_z_triangle( GLcontext *ctx,
     ffz += fdzdx;                           \
     }                                   \
     }
-#ifdef WIN32
-	#include "..\tritemp.h"
-#else
+#ifdef __MINGW32__
 	#include "tritemp.h"
+#else
+
+	#ifdef WIN32
+		#include "..\tritemp.h"
+	#else
+		#include "tritemp.h"
+	#endif
 #endif
 }
 
@@ -2231,8 +2281,8 @@ static void flat_8A8B8G8R_z_triangle( GLcontext *ctx, GLuint v0,
     //#define BYTES_PER_ROW (wmesa->xm_buffer->backimage->bytes_per_line)
 #define BYTES_PER_ROW (wmesa->ScanWidth)
 #define SETUP_CODE                  \
-    unsigned long p = PACK_8B8G8R( VB->Color[pv][0],    \
-    VB->Color[pv][1], VB->Color[pv][2] );
+    unsigned long p = PACK_8B8G8R( VB->ColorPtr->data[pv][0],    \
+    VB->ColorPtr->data[pv][1], VB->ColorPtr->data[pv][2] );
 #define INNER_LOOP( LEFT, RIGHT, Y )                    \
     {                                   \
     GLint i, len = RIGHT-LEFT;                      \
@@ -2245,10 +2295,15 @@ static void flat_8A8B8G8R_z_triangle( GLcontext *ctx, GLuint v0,
     ffz += fdzdx;                           \
     }                                   \
     }
-#ifdef WIN32
-	#include "..\tritemp.h"
-#else
+#ifdef __MINGW32__
 	#include "tritemp.h"
+#else
+
+	#ifdef WIN32
+		#include "..\tritemp.h"
+	#else
+		#include "tritemp.h"
+	#endif
 #endif
 }
 
@@ -2266,8 +2321,8 @@ static void flat_8R8G8B_z_triangle( GLcontext *ctx, GLuint v0, GLuint v1,
     //#define BYTES_PER_ROW (wmesa->xm_buffer->backimage->bytes_per_line)
 #define BYTES_PER_ROW (wmesa->ScanWidth)
 #define SETUP_CODE                  \
-    unsigned long p = PACK_8R8G8B( VB->Color[pv][0],    \
-    VB->Color[pv][1], VB->Color[pv][2] );
+    unsigned long p = PACK_8R8G8B( VB->ColorPtr->data[pv][0],    \
+    VB->ColorPtr->data[pv][1], VB->ColorPtr->data[pv][2] );
 #define INNER_LOOP( LEFT, RIGHT, Y )            \
     {                           \
     GLint i, len = RIGHT-LEFT;              \
@@ -2280,10 +2335,15 @@ static void flat_8R8G8B_z_triangle( GLcontext *ctx, GLuint v0, GLuint v1,
     ffz += fdzdx;                   \
     }                           \
     }
-#ifdef WIN32
-	#include "..\tritemp.h"
-#else
+#ifdef __MINGW32__
 	#include "tritemp.h"
+#else
+
+	#ifdef WIN32
+		#include "..\tritemp.h"
+	#else
+		#include "tritemp.h"
+	#endif
 #endif
 }
 
@@ -2301,8 +2361,8 @@ static void flat_5R6G5B_z_triangle( GLcontext *ctx, GLuint v0, GLuint v1,
     //#define BYTES_PER_ROW (wmesa->xm_buffer->backimage->bytes_per_line)
 #define BYTES_PER_ROW (wmesa->ScanWidth)
 #define SETUP_CODE                  \
-    unsigned long p = PACK_5R6G5B( VB->Color[pv][0],    \
-    VB->Color[pv][1], VB->Color[pv][2] );
+    unsigned long p = PACK_5R6G5B( VB->ColorPtr->data[pv][0],    \
+    VB->ColorPtr->data[pv][1], VB->ColorPtr->data[pv][2] );
 #define INNER_LOOP( LEFT, RIGHT, Y )            \
     {                           \
     GLint i, len = RIGHT-LEFT;              \
@@ -2315,10 +2375,15 @@ static void flat_5R6G5B_z_triangle( GLcontext *ctx, GLuint v0, GLuint v1,
     ffz += fdzdx;                   \
     }                           \
     }
-#ifdef WIN32
-	#include "..\tritemp.h"
-#else
+#ifdef __MINGW32__
 	#include "tritemp.h"
+#else
+
+	#ifdef WIN32
+		#include "..\tritemp.h"
+	#else
+		#include "tritemp.h"
+	#endif
 #endif
 }
 
@@ -2345,10 +2410,15 @@ static void smooth_8A8B8G8R_triangle( GLcontext *ctx, GLuint v0, GLuint v1,
                 ffr += fdrdx;  ffg += fdgdx;  ffb += fdbdx;         \
     }                                   \
     }
-#ifdef WIN32
-	#include "..\tritemp.h"
-#else
+#ifdef __MINGW32__
 	#include "tritemp.h"
+#else
+
+	#ifdef WIN32
+		#include "..\tritemp.h"
+	#else
+		#include "tritemp.h"
+	#endif
 #endif
 }
 
@@ -2375,10 +2445,15 @@ static void smooth_8R8G8B_triangle( GLcontext *ctx, GLuint v0, GLuint v1,
                 ffr += fdrdx;  ffg += fdgdx;  ffb += fdbdx;         \
     }                                   \
     }
-#ifdef WIN32
-	#include "..\tritemp.h"
-#else
+#ifdef __MINGW32__
 	#include "tritemp.h"
+#else
+
+	#ifdef WIN32
+		#include "..\tritemp.h"
+	#else
+		#include "tritemp.h"
+	#endif
 #endif
 }
 
@@ -2405,10 +2480,15 @@ static void smooth_5R6G5B_triangle( GLcontext *ctx, GLuint v0, GLuint v1,
                 ffr += fdrdx;  ffg += fdgdx;  ffb += fdbdx;         \
     }                                   \
     }
-#ifdef WIN32
-	#include "..\tritemp.h"
-#else
+#ifdef __MINGW32__
 	#include "tritemp.h"
+#else
+
+	#ifdef WIN32
+		#include "..\tritemp.h"
+	#else
+		#include "tritemp.h"
+	#endif
 #endif
 }
 
@@ -2426,8 +2506,8 @@ static void flat_8A8B8G8R_triangle( GLcontext *ctx, GLuint v0,
     //#define BYTES_PER_ROW (wmesa->xm_buffer->backimage->bytes_per_line)
 #define BYTES_PER_ROW (wmesa->ScanWidth)
 #define SETUP_CODE                  \
-    unsigned long p = PACK_8B8G8R( VB->Color[pv][0],    \
-    VB->Color[pv][1], VB->Color[pv][2] );
+    unsigned long p = PACK_8B8G8R( VB->ColorPtr->data[pv][0],    \
+    VB->ColorPtr->data[pv][1], VB->ColorPtr->data[pv][2] );
 #define INNER_LOOP( LEFT, RIGHT, Y )            \
     {                           \
     GLint xx;                       \
@@ -2436,10 +2516,16 @@ static void flat_8A8B8G8R_triangle( GLcontext *ctx, GLuint v0,
     *pixel = p;                 \
     }                           \
     }
-#ifdef WIN32
-	#include "..\tritemp.h"
-#else
+
+#ifdef __MINGW32__
 	#include "tritemp.h"
+#else
+
+	#ifdef WIN32
+		#include "..\tritemp.h"
+	#else
+		#include "tritemp.h"
+	#endif
 #endif
 }
 
@@ -2456,8 +2542,8 @@ static void flat_8R8G8B_triangle( GLcontext *ctx, GLuint v0, GLuint v1,
     //#define BYTES_PER_ROW (wmesa->xm_buffer->backimage->bytes_per_line)
 #define BYTES_PER_ROW (wmesa->ScanWidth)
 #define SETUP_CODE                  \
-    unsigned long p = PACK_8R8G8B( VB->Color[pv][0],    \
-    VB->Color[pv][1], VB->Color[pv][2] );
+    unsigned long p = PACK_8R8G8B( VB->ColorPtr->data[pv][0],    \
+    VB->ColorPtr->data[pv][1], VB->ColorPtr->data[pv][2] );
 #define INNER_LOOP( LEFT, RIGHT, Y )            \
     {                           \
     GLint xx;                       \
@@ -2466,10 +2552,15 @@ static void flat_8R8G8B_triangle( GLcontext *ctx, GLuint v0, GLuint v1,
     *pixel = p;                 \
     }                           \
     }
-#ifdef WIN32
-	#include "..\tritemp.h"
-#else
+#ifdef __MINGW32__
 	#include "tritemp.h"
+#else
+
+	#ifdef WIN32
+		#include "..\tritemp.h"
+	#else
+		#include "tritemp.h"
+	#endif
 #endif
 }
 
@@ -2486,8 +2577,8 @@ static void flat_5R6G5B_triangle( GLcontext *ctx, GLuint v0, GLuint v1,
     //#define BYTES_PER_ROW (wmesa->xm_buffer->backimage->bytes_per_line)
 #define BYTES_PER_ROW (wmesa->ScanWidth)
 #define SETUP_CODE                  \
-    unsigned long p = PACK_5R6G5B( VB->Color[pv][0],    \
-    VB->Color[pv][1], VB->Color[pv][2] );
+    unsigned long p = PACK_5R6G5B( VB->ColorPtr->data[pv][0],    \
+    VB->ColorPtr->data[pv][1], VB->ColorPtr->data[pv][2] );
 #define INNER_LOOP( LEFT, RIGHT, Y )            \
     {                           \
     GLint xx;                       \
@@ -2496,10 +2587,15 @@ static void flat_5R6G5B_triangle( GLcontext *ctx, GLuint v0, GLuint v1,
     *pixel = p;                 \
     }                           \
     }
-#ifdef WIN32
-	#include "..\tritemp.h"
-#else
+#ifdef __MINGW32__
 	#include "tritemp.h"
+#else
+
+	#ifdef WIN32
+		#include "..\tritemp.h"
+	#else
+		#include "tritemp.h"
+	#endif
 #endif
 }
 
@@ -2530,11 +2626,15 @@ static void smooth_ci_z_triangle( GLcontext *ctx, GLuint v0, GLuint v1,
     ffz += fdzdx;                                                   \
     }                                                               \
     }
-
-#ifdef WIN32
-	#include "..\tritemp.h"
-#else
+#ifdef __MINGW32__
 	#include "tritemp.h"
+#else
+
+	#ifdef WIN32
+		#include "..\tritemp.h"
+	#else
+		#include "tritemp.h"
+	#endif
 #endif
 }
 
@@ -2551,28 +2651,30 @@ static void flat_ci_z_triangle( GLcontext *ctx, GLuint v0, GLuint v1,
 #define PIXEL_ADDRESS(X,Y) PIXELADDR1(X,Y)
 #define PIXEL_TYPE GLubyte
 #define BYTES_PER_ROW (wmesa->ScanWidth)
-#define SETUP_CODE                                                  \
-    GLuint index = VB->Index[pv];                                   \
-    if (!VB->MonoColor) {                                           \
-    /* set the color index */                                       \
-    (*ctx->Driver.Index)( ctx, index );                         \
-    }
-#define INNER_LOOP( LEFT, RIGHT, Y )                                \
-    {                                                                   \
-    GLint i, len = RIGHT-LEFT;                                      \
-    for (i=0;i<len;i++) {                                           \
-    GLdepth z = FixedToDepth(ffz);                              \
-    if (z < zRow[i]) {                                          \
-    pRow[i] = index;                                            \
-    zRow[i] = z;                                                \
-    }                                                               \
-    ffz += fdzdx;                                                   \
-    }                                                               \
-    }
-#ifdef WIN32
-	#include "..\tritemp.h"
-#else
+#define SETUP_CODE			\
+   GLuint index = VB->IndexPtr->data[pv];	\
+   (*ctx->Driver.Index)( ctx, index );
+#define INNER_LOOP( LEFT, RIGHT, Y )	\
+   {					\
+      GLint i, len = RIGHT-LEFT;	\
+      for (i=0;i<len;i++) {		\
+         GLdepth z = FixedToDepth(ffz);	\
+         if (z < zRow[i]) {		\
+            pRow[i] = index;		\
+            zRow[i] = z;		\
+         }				\
+         ffz += fdzdx;			\
+      }					\
+   }
+#ifdef __MINGW32__
 	#include "tritemp.h"
+#else
+
+	#ifdef WIN32
+		#include "..\tritemp.h"
+	#else
+		#include "tritemp.h"
+	#endif
 #endif
 }
 
@@ -2600,10 +2702,15 @@ static void smooth_ci_triangle( GLcontext *ctx, GLuint v0, GLuint v1,
     ffi += fdidx;           \
     }                                   \
     }
-#ifdef WIN32
-	#include "..\tritemp.h"
-#else
+#ifdef __MINGW32__
 	#include "tritemp.h"
+#else
+
+	#ifdef WIN32
+		#include "..\tritemp.h"
+	#else
+		#include "tritemp.h"
+	#endif
 #endif
 }
 
@@ -2619,24 +2726,26 @@ static void flat_ci_triangle( GLcontext *ctx, GLuint v0, GLuint v1,
 #define PIXEL_ADDRESS(X,Y) PIXELADDR1(X,Y)
 #define PIXEL_TYPE GLubyte
 #define BYTES_PER_ROW (wmesa->ScanWidth)
-#define SETUP_CODE                                                  \
-    GLuint index = VB->Index[pv];                                   \
-    if (!VB->MonoColor) {                                           \
-    /* set the color index */                                       \
-    (*ctx->Driver.Index)( ctx, index );                         \
-    }
-#define INNER_LOOP( LEFT, RIGHT, Y )            \
-    {                           \
-    GLint xx;                       \
-    PIXEL_TYPE *pixel = pRow;               \
-    for (xx=LEFT;xx<RIGHT;xx++,pixel++) {       \
-    *pixel = index;                 \
-    }                           \
-    }
-#ifdef WIN32
-	#include "..\tritemp.h"
-#else
+#define SETUP_CODE			\
+   GLuint index = VB->IndexPtr->data[pv];	\
+   (*ctx->Driver.Index)( ctx, index );
+#define INNER_LOOP( LEFT, RIGHT, Y )		\
+   {						\
+      GLint xx;					\
+      PIXEL_TYPE *pixel = pRow;			\
+      for (xx=LEFT;xx<RIGHT;xx++,pixel++) {	\
+         *pixel = index;			\
+      }						\
+   }
+#ifdef __MINGW32__
 	#include "tritemp.h"
+#else
+
+	#ifdef WIN32
+		#include "..\tritemp.h"
+	#else
+		#include "tritemp.h"
+	#endif
 #endif
 }
 
@@ -2669,10 +2778,15 @@ static void smooth_DITHER8_z_triangle( GLcontext *ctx,
     ffz += fdzdx;                                                       \
     }                                                                   \
     }
-#ifdef WIN32
-	#include "..\tritemp.h"
-#else
+#ifdef __MINGW32__
 	#include "tritemp.h"
+#else
+
+	#ifdef WIN32
+		#include "..\tritemp.h"
+	#else
+		#include "tritemp.h"
+	#endif
 #endif
 }
 
@@ -2695,18 +2809,23 @@ static void flat_DITHER8_z_triangle( GLcontext *ctx, GLuint v0, GLuint v1,
     for (i=0;i<len;i++,xx++) {                                          \
     GLdepth z = FixedToDepth(ffz);                                  \
     if (z < zRow[i]) {                                              \
-    DITHER_RGB_TO_8BIT( VB->Color[pv][0],                           \
-             VB->Color[pv][1], VB->Color[pv][2], xx, yy);               \
+    DITHER_RGB_TO_8BIT( VB->ColorPtr->data[pv][0],                           \
+             VB->ColorPtr->data[pv][1], VB->ColorPtr->data[pv][2], xx, yy);               \
              pRow[i] = pixelDithered;                                       \
              zRow[i] = z;                                                   \
     }                                                                   \
     ffz += fdzdx;                                                       \
     }                                                                   \
     }
-#ifdef WIN32
-	#include "..\tritemp.h"
-#else
+#ifdef __MINGW32__
 	#include "tritemp.h"
+#else
+
+	#ifdef WIN32
+		#include "..\tritemp.h"
+	#else
+		#include "tritemp.h"
+	#endif
 #endif
 }
 
@@ -2727,15 +2846,20 @@ static void smooth_DITHER8_triangle( GLcontext *ctx, GLuint v0, GLuint v1,
     GLint xx, yy = FLIP(Y);                                             \
     PIXEL_TYPE *pixel = pRow;                                           \
     for (xx=LEFT;xx<RIGHT;xx++,pixel++) {                               \
-    DITHER_RGB_TO_8BIT( VB->Color[pv][0],   VB->Color[pv][1], VB->Color[pv][2], xx, yy);\
+    DITHER_RGB_TO_8BIT( VB->ColorPtr->data[pv][0],   VB->ColorPtr->data[pv][1], VB->ColorPtr->data[pv][2], xx, yy);\
     *pixel = pixelDithered;                                         \
     ffr += fdrdx;  ffg += fdgdx;  ffb += fdbdx;                     \
     }                                                                   \
     }
-#ifdef WIN32
-	#include "..\tritemp.h"
-#else
+#ifdef __MINGW32__
 	#include "tritemp.h"
+#else
+
+	#ifdef WIN32
+		#include "..\tritemp.h"
+	#else
+		#include "tritemp.h"
+	#endif
 #endif
 }
 
@@ -2757,15 +2881,20 @@ static void flat_DITHER8_triangle( GLcontext *ctx, GLuint v0, GLuint v1,
     GLint xx, yy = FLIP(Y);                                             \
     PIXEL_TYPE *pixel = pRow;                                           \
     for (xx=LEFT;xx<RIGHT;xx++,pixel++) {                               \
-    DITHER_RGB_TO_8BIT( VB->Color[pv][0],                               \
-             VB->Color[pv][1], VB->Color[pv][2], xx, yy);               \
+    DITHER_RGB_TO_8BIT( VB->ColorPtr->data[pv][0],                               \
+             VB->ColorPtr->data[pv][1], VB->ColorPtr->data[pv][2], xx, yy);               \
              *pixel = pixelDithered;                                            \
     }                                                                   \
     }
-#ifdef WIN32
-	#include "..\tritemp.h"
-#else
+#ifdef __MINGW32__
 	#include "tritemp.h"
+#else
+
+	#ifdef WIN32
+		#include "..\tritemp.h"
+	#else
+		#include "tritemp.h"
+	#endif
 #endif
 }
 
@@ -2877,8 +3006,14 @@ void WMesaViewport( GLcontext *ctx,
     ctx->Viewport.Height = height;
 
     /* compute scale and bias values */
-    ctx->Viewport.Sx = (GLfloat) width / 2.0F;
-    ctx->Viewport.Tx = ctx->Viewport.Sx + x;
-    ctx->Viewport.Sy = (GLfloat) height / 2.0F;
-    ctx->Viewport.Ty = ctx->Viewport.Sy + y;
+/* Pre-Keith 3.1 changes
+    ctx->Viewport.Map.m[Sx] = (GLfloat) width / 2.0F;
+    ctx->Viewport.Map.m[Tx] = ctx->Viewport.Sx + x;
+    ctx->Viewport.Map.m[Sy] = (GLfloat) height / 2.0F;
+    ctx->Viewport.Map.m[Ty] = ctx->Viewport.Sy + y;
+*/
+	ctx->Viewport.WindowMap.m[MAT_SX] = (GLfloat) width / 2.0F;
+	ctx->Viewport.WindowMap.m[MAT_TX] = ctx->Viewport.WindowMap.m[MAT_SX] + x;
+	ctx->Viewport.WindowMap.m[MAT_SY] = (GLfloat) height / 2.0F;
+	ctx->Viewport.WindowMap.m[MAT_TY] = ctx->Viewport.WindowMap.m[MAT_SY] + y;
 }
