@@ -1,8 +1,8 @@
-/* $Id: fakeglx.c,v 1.12.2.8 2000/04/24 12:42:37 brianp Exp $ */
+/* $Id: fakeglx.c,v 1.34 2000/06/08 22:50:28 brianp Exp $ */
 
 /*
  * Mesa 3-D graphics library
- * Version:  3.2
+ * Version:  3.3
  * 
  * Copyright (C) 1999-2000  Brian Paul   All Rights Reserved.
  * 
@@ -25,12 +25,9 @@
  */
 
 
-
-
-
 /*
- * A pseudo-GLX implementation to allow OpenGL/GLX programs to work with Mesa.
- * The Fake_glX*() functions implemented here are called from glxapi.c
+ * This is an emulation of the GLX API which allows Mesa/GLX-based programs
+ * to run on X servers which do not have the real GLX extension.
  *
  * Thanks to the contributors:
  *
@@ -45,27 +42,38 @@
 
 
 
-#ifdef HAVE_CONFIG_H
-#include "conf.h"
-#endif
-
-#include <stdio.h>
-#include <stdlib.h>
-#include <string.h>
-#include <X11/Xlib.h>
-#include <X11/Xutil.h>
-#include "GL/gl.h"
+#include "glxheader.h"
+#include "glxapi.h"
 #include "GL/xmesa.h"
 #include "context.h"
 #include "config.h"
-#include "fakeglx.h"
 #include "macros.h"
+#include "mmath.h"
 #include "types.h"
+#include "xfonts.h"
 #include "xmesaP.h"
 
 
+/* This indicates the client-side GLX API and GLX encoder version. */
+#define CLIENT_MAJOR_VERSION 1
+#define CLIENT_MINOR_VERSION 2
+
+/* This indicates the server-side GLX decoder version.
+ * GLX 1.3 indicates OpenGL 1.2 support
+ */
+#define SERVER_MAJOR_VERSION 1
+#define SERVER_MINOR_VERSION 3
+
+/* This is appended onto the glXGetClient/ServerString version strings. */
+#define MESA_GLX_VERSION "Mesa 3.3"
+
+/* Who implemented this GLX? */
+#define VENDOR "Brian Paul"
+
+
 
 /* Silence compiler warnings */
+extern void Fake_glXDummyFunc( void );
 void Fake_glXDummyFunc( void )
 {
    (void) kernel8;
@@ -222,7 +230,9 @@ save_glx_visual( Display *dpy, XVisualInfo *vinfo,
                  GLboolean rgbFlag, GLboolean alphaFlag, GLboolean dbFlag,
                  GLboolean stereoFlag,
                  GLint depth_size, GLint stencil_size,
-                 GLint accum_size, GLint level )
+                 GLint accumRedSize, GLint accumGreenSize,
+                 GLint accumBlueSize, GLint accumAlphaSize,
+                 GLint level )
 {
    GLboolean ximageFlag = GL_TRUE;
    XMesaVisual xmvis;
@@ -265,7 +275,10 @@ save_glx_visual( Display *dpy, XVisualInfo *vinfo,
           && (v->gl_visual->AlphaBits > 0) == alphaFlag
           && (v->gl_visual->DepthBits >= depth_size || depth_size == 0)
           && (v->gl_visual->StencilBits >= stencil_size || stencil_size == 0)
-          && (v->gl_visual->AccumBits >= accum_size || accum_size == 0)) {
+          && (v->gl_visual->AccumRedBits >= accumRedSize || accumRedSize == 0)
+          && (v->gl_visual->AccumGreenBits >= accumGreenSize || accumGreenSize == 0)
+          && (v->gl_visual->AccumBlueBits >= accumBlueSize || accumBlueSize == 0)
+          && (v->gl_visual->AccumAlphaBits >= accumAlphaSize || accumAlphaSize == 0)) {
          /* now either compare XVisualInfo pointers or visual IDs */
          if ((!comparePointers && v->visinfo->visualid == vinfo->visualid)
              || (comparePointers && v->vishandle == vinfo)) {
@@ -283,7 +296,10 @@ save_glx_visual( Display *dpy, XVisualInfo *vinfo,
 
    xmvis = XMesaCreateVisual( dpy, vinfo, rgbFlag, alphaFlag, dbFlag,
                               stereoFlag, ximageFlag,
-                              depth_size, stencil_size, accum_size, level );
+                              depth_size, stencil_size,
+                              accumRedSize, accumBlueSize,
+                              accumBlueSize, accumAlphaSize, 0, level,
+                              GLX_NONE_EXT );
    if (xmvis) {
       VisualTable[NumVisuals] = xmvis;
       NumVisuals++;
@@ -295,6 +311,11 @@ save_glx_visual( Display *dpy, XVisualInfo *vinfo,
 
 /*
  * Create a GLX visual from a regular XVisualInfo.
+ * This is called when Fake GLX is given an XVisualInfo which wasn't
+ * returned by glXChooseVisual.  Since this is the first time we're
+ * considering this visual we'll take a guess at reasonable values
+ * for depth buffer size, stencil size, accum size, etc.
+ * This is the best we can do with a client-side emulation of GLX.
  */
 static XMesaVisual
 create_glx_visual( Display *dpy, XVisualInfo *visinfo )
@@ -311,7 +332,7 @@ create_glx_visual( Display *dpy, XVisualInfo *visinfo )
                               GL_FALSE,  /* stereo */
                               0,         /* depth bits */
                               0,         /* stencil bits */
-                              0,         /* accum bits */
+                              0,0,0,0,   /* accum bits */
                               vislevel   /* level */
                             );
    }
@@ -324,9 +345,12 @@ create_glx_visual( Display *dpy, XVisualInfo *visinfo )
                               GL_FALSE,  /* alpha */
                               GL_TRUE,   /* double */
                               GL_FALSE,  /* stereo */
-                              8*sizeof(GLdepth),
-                              8*sizeof(GLstencil),
-                              8*sizeof(GLaccum),
+                              DEFAULT_SOFTWARE_DEPTH_BITS,
+                              8 * sizeof(GLstencil),
+                              8 * sizeof(GLaccum), /* r */
+                              8 * sizeof(GLaccum), /* g */
+                              8 * sizeof(GLaccum), /* b */
+                              8 * sizeof(GLaccum), /* a */
                               0          /* level */
                             );
    }
@@ -431,21 +455,6 @@ static int transparent_pixel( XMesaVisual glxvis )
 
 
 
-/*
- * Return number of bits set in n.
- */
-static int bitcount( unsigned long n )
-{
-   int bits;
-   for (bits=0; n>0; n=n>>1) {
-      if (n&1) {
-         bits++;
-      }
-   }
-   return bits;
-}
-
-
 /*
  * Try to get an X visual which matches the given arguments.
  */
@@ -479,9 +488,9 @@ static XVisualInfo *get_visual( Display *dpy, int scr,
     * 10 bits per color channel.  Mesa's limited to a max of 8 bits/channel.
     */
    if (vis && depth > 24 && (xclass==TrueColor || xclass==DirectColor)) {
-      if (bitcount(vis->red_mask) <= 8
-          && bitcount(vis->green_mask) <= 8
-          && bitcount(vis->blue_mask) <= 8) {
+      if (_mesa_bitcount((GLuint) vis->red_mask  ) <= 8 &&
+          _mesa_bitcount((GLuint) vis->green_mask) <= 8 &&
+          _mesa_bitcount((GLuint) vis->blue_mask ) <= 8) {
          return vis;
       }
       else {
@@ -843,7 +852,8 @@ static XVisualInfo *choose_x_overlay_visual( Display *dpy, int scr,
 
 
 
-XVisualInfo *Fake_glXChooseVisual( Display *dpy, int screen, int *list )
+static XVisualInfo *
+Fake_glXChooseVisual( Display *dpy, int screen, int *list )
 {
    int *parselist;
    XVisualInfo *vis;
@@ -855,11 +865,15 @@ XVisualInfo *Fake_glXChooseVisual( Display *dpy, int screen, int *list )
    GLboolean stereo_flag = GL_FALSE;
    GLint depth_size = 0;
    GLint stencil_size = 0;
-   GLint accum_size = 0;
+   GLint accumRedSize = 0;
+   GLint accumGreenSize = 0;
+   GLint accumBlueSize = 0;
+   GLint accumAlphaSize = 0;
    int level = 0;
    int visual_type = DONT_CARE;
    int trans_type = DONT_CARE;
    int trans_value = DONT_CARE;
+   GLint caveat = DONT_CARE;
 
    parselist = list;
 
@@ -922,13 +936,31 @@ XVisualInfo *Fake_glXChooseVisual( Display *dpy, int screen, int *list )
 	    stencil_size = *parselist++;
 	    break;
 	 case GLX_ACCUM_RED_SIZE:
+	    parselist++;
+            {
+               GLint size = *parselist++;
+               accumRedSize = MAX2( accumRedSize, size );
+            }
+            break;
 	 case GLX_ACCUM_GREEN_SIZE:
+	    parselist++;
+            {
+               GLint size = *parselist++;
+               accumGreenSize = MAX2( accumGreenSize, size );
+            }
+            break;
 	 case GLX_ACCUM_BLUE_SIZE:
+	    parselist++;
+            {
+               GLint size = *parselist++;
+               accumBlueSize = MAX2( accumBlueSize, size );
+            }
+            break;
 	 case GLX_ACCUM_ALPHA_SIZE:
 	    parselist++;
             {
                GLint size = *parselist++;
-               accum_size = MAX2( accum_size, size );
+               accumAlphaSize = MAX2( accumAlphaSize, size );
             }
 	    break;
 
@@ -956,6 +988,14 @@ XVisualInfo *Fake_glXChooseVisual( Display *dpy, int screen, int *list )
 	    parselist++;
 	    break;
          
+         /*
+          * GLX_EXT_visual_info extension
+          */
+         case GLX_VISUAL_CAVEAT_EXT:
+            parselist++;
+            caveat = *parselist++; /* ignored for now */
+            break;
+
 	 case None:
 	    break;
 	 default:
@@ -985,7 +1025,7 @@ XVisualInfo *Fake_glXChooseVisual( Display *dpy, int screen, int *list )
       else {
          /* Get a color index visual */
          vis = choose_x_visual( dpy, screen, rgb_flag, min_ci, visual_type );
-         accum_size = 0;
+         accumRedSize = accumGreenSize = accumBlueSize = accumAlphaSize = 0;
       }
    }
    else {
@@ -1008,9 +1048,33 @@ XVisualInfo *Fake_glXChooseVisual( Display *dpy, int screen, int *list )
    }
 
    if (vis) {
+      /* Note: we're not exactly obeying the glXChooseVisual rules here.
+       * When GLX_DEPTH_SIZE = 1 is specified we're supposed to choose the
+       * largest depth buffer size, which is 32bits/value.  However, we
+       * return 16 to maintain performance with earlier versions of Mesa.
+       */
+      if (depth_size == 1)
+         depth_size = DEFAULT_SOFTWARE_DEPTH_BITS;
+      else if (depth_size > 24)
+         depth_size = 31;
+      else if (depth_size > 16)
+         depth_size = 24;
+      /* we only support one size of stencil and accum buffers. */
+      if (stencil_size > 0)
+         stencil_size = STENCIL_BITS;
+      if (accumRedSize > 0)
+         accumRedSize = ACCUM_BITS;
+      if (accumGreenSize > 0)
+         accumGreenSize = ACCUM_BITS;
+      if (accumBlueSize > 0)
+         accumBlueSize = ACCUM_BITS;
+      if (accumAlphaSize > 0)
+         accumAlphaSize = ACCUM_BITS;
       if (!save_glx_visual( dpy, vis, rgb_flag, alpha_flag, double_flag,
-                            stereo_flag,
-                            depth_size, stencil_size, accum_size, level ))
+                            stereo_flag, depth_size, stencil_size,
+                            accumRedSize, accumGreenSize,
+                            accumBlueSize, accumAlphaSize,
+                            level ))
          return NULL;
    }
 
@@ -1020,8 +1084,9 @@ XVisualInfo *Fake_glXChooseVisual( Display *dpy, int screen, int *list )
 
 
 
-GLXContext Fake_glXCreateContext( Display *dpy, XVisualInfo *visinfo,
-                                  GLXContext share_list, Bool direct )
+static GLXContext
+Fake_glXCreateContext( Display *dpy, XVisualInfo *visinfo,
+                       GLXContext share_list, Bool direct )
 {
    XMesaVisual glxvis;
    XMesaContext xmctx;
@@ -1048,54 +1113,94 @@ GLXContext Fake_glXCreateContext( Display *dpy, XVisualInfo *visinfo,
 }
 
 
-static GLXDrawable MakeCurrent_PrevDrawable = 0;
 static GLXContext MakeCurrent_PrevContext = 0;
-static XMesaBuffer MakeCurrent_PrevBuffer = 0;
+static GLXDrawable MakeCurrent_PrevDrawable = 0;
+static GLXDrawable MakeCurrent_PrevReadable = 0;
+static XMesaBuffer MakeCurrent_PrevDrawBuffer = 0;
+static XMesaBuffer MakeCurrent_PrevReadBuffer = 0;
 
-Bool Fake_glXMakeCurrent( Display *dpy, GLXDrawable drawable, GLXContext ctx )
+/* GLX 1.3 and later */
+static Bool
+Fake_glXMakeContextCurrent( Display *dpy, GLXDrawable draw,
+                            GLXDrawable read, GLXContext ctx )
 {
-   if (ctx && drawable) {
-      XMesaBuffer buffer;
+   if (ctx && draw && read) {
+      XMesaBuffer drawBuffer, readBuffer;
+      XMesaContext xmctx = (XMesaContext) ctx;
 
-      if (drawable==MakeCurrent_PrevDrawable && ctx==MakeCurrent_PrevContext) {
-         buffer = MakeCurrent_PrevBuffer;
+      /* Find the XMesaBuffer which corresponds to the GLXDrawable 'draw' */
+      if (ctx == MakeCurrent_PrevContext
+          && draw == MakeCurrent_PrevDrawable) {
+         drawBuffer = MakeCurrent_PrevDrawBuffer;
       }
       else {
-         buffer = XMesaFindBuffer( dpy, drawable );
+         drawBuffer = XMesaFindBuffer( dpy, draw );
       }
-      if (!buffer) {
+      if (!drawBuffer) {
          /* drawable must be a new window! */
-         buffer = XMesaCreateWindowBuffer2( ctx->xm_visual, drawable, ctx );
-         if (!buffer) {
+         drawBuffer = XMesaCreateWindowBuffer2( xmctx->xm_visual, draw, (XMesaContext) ctx );
+         if (!drawBuffer) {
             /* Out of memory, or context/drawable depth mismatch */
             return False;
          }
       }
+
+      /* Find the XMesaBuffer which corresponds to the GLXDrawable 'read' */
+      if (ctx == MakeCurrent_PrevContext
+          && read == MakeCurrent_PrevReadable) {
+         readBuffer = MakeCurrent_PrevReadBuffer;
+      }
+      else {
+         readBuffer = XMesaFindBuffer( dpy, read );
+      }
+      if (!readBuffer) {
+         /* drawable must be a new window! */
+         readBuffer = XMesaCreateWindowBuffer2( xmctx->xm_visual, read, (XMesaContext) ctx );
+         if (!readBuffer) {
+            /* Out of memory, or context/drawable depth mismatch */
+            return False;
+         }
+      }
+
       MakeCurrent_PrevContext = ctx;
-      MakeCurrent_PrevDrawable = drawable;
-      MakeCurrent_PrevBuffer = buffer;
+      MakeCurrent_PrevDrawable = draw;
+      MakeCurrent_PrevReadable = read;
+      MakeCurrent_PrevDrawBuffer = drawBuffer;
+      MakeCurrent_PrevReadBuffer = readBuffer;
 
       /* Now make current! */
-      return (Bool) XMesaMakeCurrent( (XMesaContext) ctx, buffer );
+      return (Bool) XMesaMakeCurrent2((XMesaContext) ctx, drawBuffer, readBuffer);
    }
-   else if (!ctx && !drawable) {
+   else if (!ctx && !draw && !read) {
       /* release current context w/out assigning new one. */
       XMesaMakeCurrent( NULL, NULL );
       MakeCurrent_PrevContext = 0;
       MakeCurrent_PrevDrawable = 0;
-      MakeCurrent_PrevBuffer = 0;
+      MakeCurrent_PrevReadable = 0;
+      MakeCurrent_PrevDrawBuffer = 0;
+      MakeCurrent_PrevReadBuffer = 0;
       return True;
    }
    else {
-      /* ctx XOR drawable is NULL, this is an error */
+      /* The args must either all be non-zero or all zero.
+       * This is an error.
+       */
       return False;
    }
 }
 
 
 
-GLXPixmap Fake_glXCreateGLXPixmap( Display *dpy, XVisualInfo *visinfo,
-                                   Pixmap pixmap )
+static Bool
+Fake_glXMakeCurrent( Display *dpy, GLXDrawable drawable, GLXContext ctx )
+{
+   return Fake_glXMakeContextCurrent( dpy, drawable, drawable, ctx );
+}
+
+
+
+static GLXPixmap
+Fake_glXCreateGLXPixmap( Display *dpy, XVisualInfo *visinfo, Pixmap pixmap )
 {
    XMesaVisual v;
    XMesaBuffer b;
@@ -1118,9 +1223,9 @@ GLXPixmap Fake_glXCreateGLXPixmap( Display *dpy, XVisualInfo *visinfo,
 
 
 #ifdef GLX_MESA_pixmap_colormap
-
-GLXPixmap Fake_glXCreateGLXPixmapMESA( Display *dpy, XVisualInfo *visinfo,
-                                       Pixmap pixmap, Colormap cmap )
+static GLXPixmap
+Fake_glXCreateGLXPixmapMESA( Display *dpy, XVisualInfo *visinfo,
+                             Pixmap pixmap, Colormap cmap )
 {
    XMesaVisual v;
    XMesaBuffer b;
@@ -1140,11 +1245,11 @@ GLXPixmap Fake_glXCreateGLXPixmapMESA( Display *dpy, XVisualInfo *visinfo,
    }
    return b->frontbuffer;
 }
-
 #endif
 
 
-void Fake_glXDestroyGLXPixmap( Display *dpy, GLXPixmap pixmap )
+static void
+Fake_glXDestroyGLXPixmap( Display *dpy, GLXPixmap pixmap )
 {
    XMesaBuffer b = XMesaFindBuffer(dpy, pixmap);
    if (b) {
@@ -1156,8 +1261,10 @@ void Fake_glXDestroyGLXPixmap( Display *dpy, GLXPixmap pixmap )
 }
 
 
-void Fake_glXCopyContext( Display *dpy, GLXContext src, GLXContext dst,
-                          unsigned long mask )
+
+static void
+Fake_glXCopyContext( Display *dpy, GLXContext src, GLXContext dst,
+                     unsigned long mask )
 {
    XMesaContext xm_src = (XMesaContext) src;
    XMesaContext xm_dst = (XMesaContext) dst;
@@ -1167,7 +1274,8 @@ void Fake_glXCopyContext( Display *dpy, GLXContext src, GLXContext dst,
 
 
 
-Bool Fake_glXQueryExtension( Display *dpy, int *errorb, int *event )
+static Bool
+Fake_glXQueryExtension( Display *dpy, int *errorb, int *event )
 {
    /* Mesa's GLX isn't really an X extension but we try to act like one. */
    (void) dpy;
@@ -1177,6 +1285,7 @@ Bool Fake_glXQueryExtension( Display *dpy, int *errorb, int *event )
 }
 
 
+extern void _kw_ungrab_all( Display *dpy );
 void _kw_ungrab_all( Display *dpy )
 {
    XUngrabPointer( dpy, CurrentTime );
@@ -1184,19 +1293,23 @@ void _kw_ungrab_all( Display *dpy )
 }
 
 
-void Fake_glXDestroyContext( Display *dpy, GLXContext ctx )
+static void
+Fake_glXDestroyContext( Display *dpy, GLXContext ctx )
 {
    (void) dpy;
    MakeCurrent_PrevContext = 0;
    MakeCurrent_PrevDrawable = 0;
-   MakeCurrent_PrevBuffer = 0;
+   MakeCurrent_PrevReadable = 0;
+   MakeCurrent_PrevDrawBuffer = 0;
+   MakeCurrent_PrevReadBuffer = 0;
    XMesaDestroyContext( (XMesaContext) ctx );
    XMesaGarbageCollect();
 }
 
 
 
-Bool Fake_glXIsDirect( Display *dpy, GLXContext ctx )
+static Bool
+Fake_glXIsDirect( Display *dpy, GLXContext ctx )
 {
    (void) dpy;
    return ((XMesaContext) ctx)->direct;
@@ -1204,7 +1317,8 @@ Bool Fake_glXIsDirect( Display *dpy, GLXContext ctx )
 
 
 
-void Fake_glXSwapBuffers( Display *dpy, GLXDrawable drawable )
+static void
+Fake_glXSwapBuffers( Display *dpy, GLXDrawable drawable )
 {
    XMesaBuffer buffer = XMesaFindBuffer( dpy, drawable );
 
@@ -1217,8 +1331,11 @@ void Fake_glXSwapBuffers( Display *dpy, GLXDrawable drawable )
 }
 
 
-void Fake_glXCopySubBufferMESA( Display *dpy, GLXDrawable drawable,
-				int x, int y, int width, int height )
+
+#ifdef GLX_MESA_copy_sub_buffer
+static void
+Fake_glXCopySubBufferMESA( Display *dpy, GLXDrawable drawable,
+                           int x, int y, int width, int height )
 {
    XMesaBuffer buffer = XMesaFindBuffer( dpy, drawable );
    if (buffer) {
@@ -1228,15 +1345,18 @@ void Fake_glXCopySubBufferMESA( Display *dpy, GLXDrawable drawable,
       fprintf(stderr, "Mesa Warning: glXCopySubBufferMESA: invalid drawable\n");
    }
 }
+#endif
 
 
 
-Bool Fake_glXQueryVersion( Display *dpy, int *maj, int *min )
+static Bool
+Fake_glXQueryVersion( Display *dpy, int *maj, int *min )
 {
    (void) dpy;
    /* Return GLX version, not Mesa version */
-   *maj = 1;
-   *min = 1;
+   assert(CLIENT_MAJOR_VERSION == SERVER_MAJOR_VERSION);
+   *maj = CLIENT_MAJOR_VERSION;
+   *min = MIN2( CLIENT_MINOR_VERSION, SERVER_MINOR_VERSION );
    return True;
 }
 
@@ -1245,8 +1365,9 @@ Bool Fake_glXQueryVersion( Display *dpy, int *maj, int *min )
 /*
  * Query the GLX attributes of the given XVisualInfo.
  */
-int Fake_glXGetConfig( Display *dpy, XVisualInfo *visinfo,
-                       int attrib, int *value )
+static int
+Fake_glXGetConfig( Display *dpy, XVisualInfo *visinfo,
+                   int attrib, int *value )
 {
    XMesaVisual glxvis;
 
@@ -1313,15 +1434,16 @@ int Fake_glXGetConfig( Display *dpy, XVisualInfo *visinfo,
 	 *value = glxvis->gl_visual->StencilBits;
 	 return 0;
       case GLX_ACCUM_RED_SIZE:
+	 *value = glxvis->gl_visual->AccumRedBits;
+	 return 0;
       case GLX_ACCUM_GREEN_SIZE:
+	 *value = glxvis->gl_visual->AccumGreenBits;
+	 return 0;
       case GLX_ACCUM_BLUE_SIZE:
-	 *value = glxvis->gl_visual->AccumBits;
+	 *value = glxvis->gl_visual->AccumBlueBits;
 	 return 0;
       case GLX_ACCUM_ALPHA_SIZE:
-         if (glxvis->gl_visual->AlphaBits > 0)
-            *value = glxvis->gl_visual->AccumBits;
-         else
-            *value = 0;
+         *value = glxvis->gl_visual->AccumAlphaBits;
 	 return 0;
 
       /*
@@ -1378,6 +1500,17 @@ int Fake_glXGetConfig( Display *dpy, XVisualInfo *visinfo,
          /* undefined */
          return 0;
 
+      /*
+       * GLX_EXT_visual_info extension
+       */
+      case GLX_VISUAL_CAVEAT_EXT:
+         /* test for zero, just in case */
+         if (glxvis->VisualCaveat > 0)
+            *value = glxvis->VisualCaveat;
+         else
+            *value = GLX_NONE_EXT;
+         return 0;
+
       /*
        * Extensions
        */
@@ -1388,26 +1521,8 @@ int Fake_glXGetConfig( Display *dpy, XVisualInfo *visinfo,
 
 
 
-GLXContext Fake_glXGetCurrentContext( void )
-{
-   return (GLXContext) XMesaGetCurrentContext();
-}
-
-
-
-GLXDrawable Fake_glXGetCurrentDrawable( void )
-{
-   XMesaBuffer b = XMesaGetCurrentBuffer();
-   if (b) {
-      return b->frontbuffer;
-   }
-   else {
-      return 0;
-   }
-}
-
-
-void Fake_glXWaitGL( void )
+static void
+Fake_glXWaitGL( void )
 {
    XMesaContext xmesa = XMesaGetCurrentContext();
    XMesaFlush( xmesa );
@@ -1415,7 +1530,8 @@ void Fake_glXWaitGL( void )
 
 
 
-void Fake_glXWaitX( void )
+static void
+Fake_glXWaitX( void )
 {
    XMesaContext xmesa = XMesaGetCurrentContext();
    XMesaFlush( xmesa );
@@ -1430,16 +1546,17 @@ static const char *get_extensions( void )
 #ifdef FX
    const char *fx = getenv("MESA_GLX_FX");
    if (fx && fx[0] != 'd') {
-      return "GLX_MESA_pixmap_colormap GLX_EXT_visual_info GLX_MESA_release_buffers GLX_MESA_copy_sub_buffer GLX_SGI_video_sync GLX_MESA_set_3dfx_mode GLX_ARB_get_proc_address";
+      return "GLX_MESA_pixmap_colormap GLX_EXT_visual_info GLX_EXT_visual_rating GLX_MESA_release_buffers GLX_MESA_copy_sub_buffer GLX_SGI_video_sync GLX_MESA_set_3dfx_mode GLX_ARB_get_proc_address";
    }
 #endif
-   return "GLX_MESA_pixmap_colormap GLX_EXT_visual_info GLX_MESA_release_buffers GLX_MESA_copy_sub_buffer GLX_SGI_video_sync GLX_ARB_get_proc_address";
+   return "GLX_MESA_pixmap_colormap GLX_EXT_visual_info GLX_EXT_visual_rating GLX_MESA_release_buffers GLX_MESA_copy_sub_buffer GLX_SGI_video_sync GLX_ARB_get_proc_address";
 }
 
 
 
 /* GLX 1.1 and later */
-const char *Fake_glXQueryExtensionsString( Display *dpy, int screen )
+static const char *
+Fake_glXQueryExtensionsString( Display *dpy, int screen )
 {
    (void) dpy;
    (void) screen;
@@ -1449,10 +1566,12 @@ const char *Fake_glXQueryExtensionsString( Display *dpy, int screen )
 
 
 /* GLX 1.1 and later */
-const char *Fake_glXQueryServerString( Display *dpy, int screen, int name )
+static const char *
+Fake_glXQueryServerString( Display *dpy, int screen, int name )
 {
-   static char *vendor = "Brian Paul";
-   static char *version = "1.1 Mesa 3.2";
+   static char version[1000];
+   sprintf(version, "%d.%d %s", SERVER_MAJOR_VERSION, SERVER_MINOR_VERSION,
+           MESA_GLX_VERSION);
 
    (void) dpy;
    (void) screen;
@@ -1461,7 +1580,7 @@ const char *Fake_glXQueryServerString( Display *dpy, int screen, int name )
       case GLX_EXTENSIONS:
          return get_extensions();
       case GLX_VENDOR:
-	 return vendor;
+	 return VENDOR;
       case GLX_VERSION:
 	 return version;
       default:
@@ -1472,10 +1591,12 @@ const char *Fake_glXQueryServerString( Display *dpy, int screen, int name )
 
 
 /* GLX 1.1 and later */
-const char *Fake_glXGetClientString( Display *dpy, int name )
+static const char *
+Fake_glXGetClientString( Display *dpy, int name )
 {
-   static char *vendor = "Brian Paul";
-   static char *version = "1.1 Mesa 3.2";
+   static char version[1000];
+   sprintf(version, "%d.%d %s", CLIENT_MAJOR_VERSION, CLIENT_MINOR_VERSION,
+           MESA_GLX_VERSION);
 
    (void) dpy;
 
@@ -1483,7 +1604,7 @@ const char *Fake_glXGetClientString( Display *dpy, int name )
       case GLX_EXTENSIONS:
          return get_extensions();
       case GLX_VENDOR:
-	 return vendor;
+	 return VENDOR;
       case GLX_VERSION:
 	 return version;
       default:
@@ -1493,11 +1614,176 @@ const char *Fake_glXGetClientString( Display *dpy, int name )
 
 
 
+/*
+ * GLX 1.3 and later
+ */
+
+static GLXFBConfig *
+Fake_glXChooseFBConfig( Display *dpy, int screen,
+                        const int *attribList, int *nitems )
+{
+   (void) dpy;
+   (void) screen;
+   (void) attribList;
+   (void) nitems;
+   return 0;
+}
+
+
+static int
+Fake_glXGetFBConfigAttrib( Display *dpy, GLXFBConfig config,
+                           int attribute, int *value )
+{
+   (void) dpy;
+   (void) config;
+   (void) attribute;
+   (void) value;
+   return 0;
+}
+
+
+static GLXFBConfig *
+Fake_glXGetFBConfigs( Display *dpy, int screen, int *nelements )
+{
+   (void) dpy;
+   (void) screen;
+   (void) nelements;
+   return 0;
+}
+
+
+static XVisualInfo *
+Fake_glXGetVisualFromFBConfig( Display *dpy, GLXFBConfig config )
+{
+   (void) dpy;
+   (void) config;
+   return 0;
+}
+
+
+static GLXWindow
+Fake_glXCreateWindow( Display *dpy, GLXFBConfig config, Window win,
+                      const int *attribList )
+{
+   (void) dpy;
+   (void) config;
+   (void) win;
+   (void) attribList;
+   return 0;
+}
+
+
+static void
+Fake_glXDestroyWindow( Display *dpy, GLXWindow window )
+{
+   (void) dpy;
+   (void) window;
+   return;
+}
+
+
+static GLXPixmap
+Fake_glXCreatePixmap( Display *dpy, GLXFBConfig config, Pixmap pixmap,
+                      const int *attribList )
+{
+   (void) dpy;
+   (void) config;
+   (void) pixmap;
+   (void) attribList;
+   return 0;
+}
+
+
+static void
+Fake_glXDestroyPixmap( Display *dpy, GLXPixmap pixmap )
+{
+   (void) dpy;
+   (void) pixmap;
+   return;
+}
+
+
+static GLXPbuffer
+Fake_glXCreatePbuffer( Display *dpy, GLXFBConfig config,
+                       const int *attribList )
+{
+   (void) dpy;
+   (void) config;
+   (void) attribList;
+   return 0;
+}
+
+
+static void
+Fake_glXDestroyPbuffer( Display *dpy, GLXPbuffer pbuf )
+{
+   (void) dpy;
+   (void) pbuf;
+}
+
+
+static void
+Fake_glXQueryDrawable( Display *dpy, GLXDrawable draw, int attribute,
+                       unsigned int *value )
+{
+   (void) dpy;
+   (void) draw;
+   (void) attribute;
+   (void) value;
+}
+
+
+static GLXContext
+Fake_glXCreateNewContext( Display *dpy, GLXFBConfig config,
+                          int renderType, GLXContext shareList, Bool direct )
+{
+   (void) dpy;
+   (void) config;
+   (void) renderType;
+   (void) shareList;
+   (void) direct;
+   return 0;
+}
+
+
+static int
+Fake_glXQueryContext( Display *dpy, GLXContext ctx, int attribute, int *value )
+{
+   (void) dpy;
+   (void) ctx;
+   (void) attribute;
+   (void) value;
+   return 0;
+}
+
+
+static void
+Fake_glXSelectEvent( Display *dpy, GLXDrawable drawable, unsigned long mask )
+{
+   (void) dpy;
+   (void) drawable;
+   (void) mask;
+}
+
+
+static void
+Fake_glXGetSelectedEvent( Display *dpy, GLXDrawable drawable,
+                          unsigned long *mask )
+{
+   (void) dpy;
+   (void) drawable;
+   (void) mask;
+}
+
+
+
+#ifdef GLX_MESA_release_buffers
 /*
  * Release the depth, stencil, accum buffers attached to a GLXDrawable
  * (a window or pixmap) prior to destroying the GLXDrawable.
  */
-Bool Fake_glXReleaseBuffersMESA( Display *dpy, GLXDrawable d )
+static Bool
+Fake_glXReleaseBuffersMESA( Display *dpy, GLXDrawable d )
 {
    XMesaBuffer b = XMesaFindBuffer(dpy, d);
    if (b) {
@@ -1506,74 +1792,127 @@ Bool Fake_glXReleaseBuffersMESA( Display *dpy, GLXDrawable d )
    }
    return False;
 }
+#endif
 
 
-/*
- * GLX_MESA_set_3dfx_mode
- */
-GLboolean Fake_glXSet3DfxModeMESA( GLint mode )
+
+#ifdef GLX_MESA_set_3dfx_mode
+static GLboolean
+Fake_glXSet3DfxModeMESA( GLint mode )
 {
    return XMesaSetFXmode( mode );
 }
+#endif
 
 
 
-void (*Fake_glXGetProcAddress( const GLubyte *procName ))()
+/*
+ * GLX_SGI_video_sync
+ */
+
+#ifdef GLX_SGI_video_sync
+
+static int
+Fake_glXGetVideoSyncSGI(unsigned int *count)
 {
-   typedef void (*gl_function)();
-   struct proc {
-      const char *name;
-      gl_function address;
-   };
-   static struct proc procTable[] = {
-      /* GLX 1.0 functions */
-      { "glXChooseVisual", (gl_function) glXChooseVisual },
-      { "glXCopyContext", (gl_function) glXCopyContext },
-      { "glXCreateContext", (gl_function) glXCreateContext },
-      { "glXCreateGLXPixmap", (gl_function) glXCreateGLXPixmap },
-      { "glXDestroyContext", (gl_function) glXDestroyContext },
-      { "glXDestroyGLXPixmap", (gl_function) glXDestroyGLXPixmap },
-      { "glXGetConfig", (gl_function) glXGetConfig },
-      { "glXGetCurrentContext", (gl_function) glXGetCurrentContext },
-      { "glXGetCurrentDrawable", (gl_function) glXGetCurrentDrawable },
-      { "glXIsDirect", (gl_function) glXIsDirect },
-      { "glXMakeCurrent", (gl_function) glXMakeCurrent },
-      { "glXQueryExtension", (gl_function) glXQueryExtension },
-      { "glXQueryVersion", (gl_function) glXQueryVersion },
-      { "glXSwapBuffers", (gl_function) glXSwapBuffers },
-      { "glXUseXFont", (gl_function) glXUseXFont },
-      { "glXWaitGL", (gl_function) glXWaitGL },
-      { "glXWaitX", (gl_function) glXWaitX },
-
-      /* GLX 1.1 functions */
-      { "glXGetClientString", (gl_function) glXGetClientString },
-      { "glXQueryExtensionsString", (gl_function) glXQueryExtensionsString },
-      { "glXQueryServerString", (gl_function) glXQueryServerString },
-
-      /* Extensions */
-      { "glXGetProcAddressARB", (gl_function) glXGetProcAddressARB },
-      { "glXCreateGLXPixmapMESA", (gl_function) glXCreateGLXPixmapMESA },
-      { "glXReleaseBuffersMESA", (gl_function) glXReleaseBuffersMESA },
-      { "glXCopySubBufferMESA", (gl_function) glXCopySubBufferMESA },
-      { "glXSet3DfxModeMESA", (gl_function) glXSet3DfxModeMESA },
-      { "glXGetVideoSyncSGI", (gl_function) glXGetVideoSyncSGI },
-      { "glXWaitVideoSyncSGI", (gl_function) glXWaitVideoSyncSGI },
-
-      { NULL, NULL }  /* end of list token */
-   };
-   GLuint i;
-
-   /* First, look for core library functions */
-   gl_function f = (gl_function) gl_get_proc_address(procName);
-   if (f)
-      return f;
-
-   /* Second, look for GLX funtion */
-   for (i = 0; procTable[i].address; i++) {
-      if (strcmp((const char *) procName, procTable[i].name) == 0)
-	  return procTable[i].address;
-   }
-
-   return NULL;
+   return 0;
 }
 
+
+static int
+Fake_glXWaitVideoSyncSGI(int divisor, int remainder, unsigned int *count)
+{
+   return 0;
+}
+
+#endif
+
+
+
+extern struct _glxapi_table *_mesa_GetGLXDispatchTable(void);
+struct _glxapi_table *_mesa_GetGLXDispatchTable(void)
+{
+   static struct _glxapi_table glx;
+
+   /* be sure our dispatch table size <= libGL's table */
+   {
+      int size = sizeof(struct _glxapi_table) / sizeof(void *);
+      (void) size;
+      assert(_glxapi_get_dispatch_table_size() >= size);
+   }
+
+   /* initialize the whole table to no-ops */
+   _glxapi_set_no_op_table(&glx);
+
+   /* now initialize the table with the functions I implement */
+   glx.ChooseVisual = Fake_glXChooseVisual;
+   glx.CopyContext = Fake_glXCopyContext;
+   glx.CreateContext = Fake_glXCreateContext;
+   glx.CreateGLXPixmap = Fake_glXCreateGLXPixmap;
+   glx.DestroyContext = Fake_glXDestroyContext;
+   glx.DestroyGLXPixmap = Fake_glXDestroyGLXPixmap;
+   glx.GetConfig = Fake_glXGetConfig;
+   /*glx.GetCurrentContext = Fake_glXGetCurrentContext;*/
+   /*glx.GetCurrentDrawable = Fake_glXGetCurrentDrawable;*/
+   glx.IsDirect = Fake_glXIsDirect;
+   glx.MakeCurrent = Fake_glXMakeCurrent;
+   glx.QueryExtension = Fake_glXQueryExtension;
+   glx.QueryVersion = Fake_glXQueryVersion;
+   glx.SwapBuffers = Fake_glXSwapBuffers;
+   glx.UseXFont = Fake_glXUseXFont;
+   glx.WaitGL = Fake_glXWaitGL;
+   glx.WaitX = Fake_glXWaitX;
+
+#ifdef GLX_VERSION_1_1
+   glx.GetClientString = Fake_glXGetClientString;
+   glx.QueryExtensionsString = Fake_glXQueryExtensionsString;
+   glx.QueryServerString = Fake_glXQueryServerString;
+#endif
+
+#ifdef GLX_VERSION_1_2
+   /*glx.GetCurrentDisplay = Fake_glXGetCurrentDisplay;*/
+#endif
+
+#ifdef GLX_VERSION_1_3
+   glx.ChooseFBConfig = Fake_glXChooseFBConfig;
+   glx.CreateNewContext = Fake_glXCreateNewContext;
+   glx.CreatePbuffer = Fake_glXCreatePbuffer;
+   glx.CreatePixmap = Fake_glXCreatePixmap;
+   glx.CreateWindow = Fake_glXCreateWindow;
+   glx.DestroyPbuffer = Fake_glXDestroyPbuffer;
+   glx.DestroyPixmap = Fake_glXDestroyPixmap;
+   glx.DestroyWindow = Fake_glXDestroyWindow;
+   /*glx.GetCurrentReadDrawable = Fake_glXGetCurrentReadDrawable;*/
+   glx.GetFBConfigAttrib = Fake_glXGetFBConfigAttrib;
+   glx.GetFBConfigs = Fake_glXGetFBConfigs;
+   glx.GetSelectedEvent = Fake_glXGetSelectedEvent;
+   glx.GetVisualFromFBConfig = Fake_glXGetVisualFromFBConfig;
+   glx.MakeContextCurrent = Fake_glXMakeContextCurrent;
+   glx.QueryContext = Fake_glXQueryContext;
+   glx.QueryDrawable = Fake_glXQueryDrawable;
+   glx.SelectEvent = Fake_glXSelectEvent;
+#endif
+
+#ifdef GLX_SGI_video_sync
+   glx.GetVideoSyncSGI = Fake_glXGetVideoSyncSGI;
+   glx.WaitVideoSyncSGI = Fake_glXWaitVideoSyncSGI;
+#endif
+
+#ifdef GLX_MESA_copy_sub_buffer
+   glx.CopySubBufferMESA = Fake_glXCopySubBufferMESA;
+#endif
+
+#ifdef GLX_MESA_release_buffers
+   glx.ReleaseBuffersMESA = Fake_glXReleaseBuffersMESA;
+#endif
+
+#ifdef GLX_MESA_pixmap_colormap
+   glx.CreateGLXPixmapMESA = Fake_glXCreateGLXPixmapMESA;
+#endif
+
+#ifdef GLX_MESA_set_3dfx_mode
+   glx.Set3DfxModeMESA = Fake_glXSet3DfxModeMESA;
+#endif
+
+   return &glx;
+}
