@@ -1,37 +1,27 @@
-/* $Id: xmesa3.c,v 3.2 1998/07/26 16:46:56 brianp Exp $ */
+/* $Id: xmesa3.c,v 1.8 2000/07/15 03:15:40 brianp Exp $ */
 
 /*
  * Mesa 3-D graphics library
- * Version:  3.0
- * Copyright (C) 1995-1998  Brian Paul
- *
- * This library is free software; you can redistribute it and/or
- * modify it under the terms of the GNU Library General Public
- * License as published by the Free Software Foundation; either
- * version 2 of the License, or (at your option) any later version.
- *
- * This library is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- * Library General Public License for more details.
- *
- * You should have received a copy of the GNU Library General Public
- * License along with this library; if not, write to the Free
- * Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
- */
-
-
-/*
- * $Log: xmesa3.c,v $
- * Revision 3.2  1998/07/26 16:46:56  brianp
- * disabled use of X stippled lines
- *
- * Revision 3.1  1998/07/17 03:38:37  brianp
- * added struct timespec declaration to silence a compiler warning
- *
- * Revision 3.0  1998/01/31 21:08:31  brianp
- * initial rev
- *
+ * Version:  3.3
+ * 
+ * Copyright (C) 1999  Brian Paul   All Rights Reserved.
+ * 
+ * Permission is hereby granted, free of charge, to any person obtaining a
+ * copy of this software and associated documentation files (the "Software"),
+ * to deal in the Software without restriction, including without limitation
+ * the rights to use, copy, modify, merge, publish, distribute, sublicense,
+ * and/or sell copies of the Software, and to permit persons to whom the
+ * Software is furnished to do so, subject to the following conditions:
+ * 
+ * The above copyright notice and this permission notice shall be included
+ * in all copies or substantial portions of the Software.
+ * 
+ * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
+ * OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
+ * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL
+ * BRIAN PAUL BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN
+ * AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
+ * CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
  */
 
 
@@ -44,13 +34,7 @@
  */
 
 
-struct timespec;  /* this silences a compiler warning on several systems */
-
-#include <sys/time.h>
-#include <assert.h>
-#include <stdlib.h>
-#include <stdio.h>
-#include <X11/Xlib.h>
+#include "glxheader.h"
 #include "depth.h"
 #include "macros.h"
 #include "vb.h"
@@ -59,7 +43,6 @@ struct timespec;  /* this silences a compiler warning on several systems */
 
 
 
-
 /**********************************************************************/
 /***                    Point rendering                             ***/
 /**********************************************************************/
@@ -71,54 +54,36 @@ struct timespec;  /* this silences a compiler warning on several systems */
 static void draw_points_ANY_pixmap( GLcontext *ctx, GLuint first, GLuint last )
 {
    XMesaContext xmesa = (XMesaContext) ctx->DriverCtx;
-   Display *dpy = xmesa->xm_visual->display;
-   Drawable buffer = xmesa->xm_buffer->buffer;
-   GC gc = xmesa->xm_buffer->gc2;
+   XMesaDisplay *dpy = xmesa->xm_visual->display;
+   XMesaDrawable buffer = xmesa->xm_buffer->buffer;
+   XMesaGC gc = xmesa->xm_buffer->gc2;
    struct vertex_buffer *VB = ctx->VB;
    register GLuint i;
 
-   if (VB->MonoColor) {
-      /* all same color */
-      XPoint p[VB_SIZE];
-      int n = 0;
+   if (xmesa->xm_visual->gl_visual->RGBAflag) {
+      /* RGB mode */
       for (i=first;i<=last;i++) {
          if (VB->ClipMask[i]==0) {
-            p[n].x =       (GLint) VB->Win[i][0];
-            p[n].y = FLIP( (GLint) VB->Win[i][1] );
-            n++;
+            register int x, y;
+	    const GLubyte *color = VB->ColorPtr->data[i];
+            unsigned long pixel = xmesa_color_to_pixel( xmesa,
+                          color[0], color[1], color[2], color[3], xmesa->pixelformat);
+            XMesaSetForeground( dpy, gc, pixel );
+            x =                         (GLint) VB->Win.data[i][0];
+            y = FLIP( xmesa->xm_buffer, (GLint) VB->Win.data[i][1] );
+            XMesaDrawPoint( dpy, buffer, gc, x, y);
          }
       }
-      XDrawPoints( dpy, buffer, xmesa->xm_buffer->gc1, p, n, CoordModeOrigin );
    }
    else {
-      /* all different colors */
-      if (xmesa->xm_visual->gl_visual->RGBAflag) {
-         /* RGB mode */
-         for (i=first;i<=last;i++) {
-            if (VB->ClipMask[i]==0) {
-               register int x, y;
-               unsigned long pixel = xmesa_color_to_pixel( xmesa,
-                                                VB->Color[i][0],
-                                                VB->Color[i][1],
-                                                VB->Color[i][2],
-                                                VB->Color[i][3] );
-               XSetForeground( dpy, gc, pixel );
-               x =       (GLint) VB->Win[i][0];
-               y = FLIP( (GLint) VB->Win[i][1] );
-               XDrawPoint( dpy, buffer, gc, x, y);
-            }
-         }
-      }
-      else {
-         /* Color index mode */
-         for (i=first;i<=last;i++) {
-            if (VB->ClipMask[i]==0) {
-               register int x, y;
-               XSetForeground( dpy, gc, VB->Index[i] );
-               x =       (GLint) VB->Win[i][0];
-               y = FLIP( (GLint) VB->Win[i][1] );
-               XDrawPoint( dpy, buffer, gc, x, y);
-            }
+      /* Color index mode */
+      for (i=first;i<=last;i++) {
+         if (VB->ClipMask[i]==0) {
+            register int x, y;
+            XMesaSetForeground( dpy, gc, VB->IndexPtr->data[i] );
+            x =                         (GLint) VB->Win.data[i][0];
+            y = FLIP( xmesa->xm_buffer, (GLint) VB->Win.data[i][1] );
+            XMesaDrawPoint( dpy, buffer, gc, x, y);
          }
       }
    }
@@ -163,28 +128,25 @@ static void flat_pixmap_line( GLcontext *ctx,
    XMesaContext xmesa = (XMesaContext) ctx->DriverCtx;
    struct vertex_buffer *VB = ctx->VB;
    register int x0, y0, x1, y1;
-   GC gc;
-   if (VB->MonoColor) {
-      gc = xmesa->xm_buffer->gc1;  /* use current color */
+   XMesaGC gc;
+   unsigned long pixel;
+   if (xmesa->xm_visual->gl_visual->RGBAflag) {
+      const GLubyte *color = VB->ColorPtr->data[pv];
+      pixel = xmesa_color_to_pixel( xmesa, color[0], color[1], color[2], color[3],
+                                    xmesa->pixelformat );
    }
    else {
-      unsigned long pixel;
-      if (xmesa->xm_visual->gl_visual->RGBAflag) {
-         pixel = xmesa_color_to_pixel( xmesa,
-                                       VB->Color[pv][0], VB->Color[pv][1],
-                                       VB->Color[pv][2], VB->Color[pv][3] );
-      }
-      else {
-         pixel = VB->Index[pv];
-      }
-      gc = xmesa->xm_buffer->gc2;
-      XSetForeground( xmesa->display, gc, pixel );
+      pixel = VB->IndexPtr->data[pv];
    }
-   x0 =       (GLint) VB->Win[vert0][0];
-   y0 = FLIP( (GLint) VB->Win[vert0][1] );
-   x1 =       (GLint) VB->Win[vert1][0];
-   y1 = FLIP( (GLint) VB->Win[vert1][1] );
-   XDrawLine( xmesa->display, xmesa->xm_buffer->buffer, gc, x0, y0, x1, y1 );
+   gc = xmesa->xm_buffer->gc2;
+   XMesaSetForeground( xmesa->display, gc, pixel );
+
+   x0 =                         (GLint) VB->Win.data[vert0][0];
+   y0 = FLIP( xmesa->xm_buffer, (GLint) VB->Win.data[vert0][1] );
+   x1 =                         (GLint) VB->Win.data[vert1][0];
+   y1 = FLIP( xmesa->xm_buffer, (GLint) VB->Win.data[vert1][1] );
+   XMesaDrawLine( xmesa->display, xmesa->xm_buffer->buffer, gc,
+		  x0, y0, x1, y1 );
 }
 
 
@@ -196,14 +158,14 @@ static void flat_TRUECOLOR_line( GLcontext *ctx,
                                  GLuint vert0, GLuint vert1, GLuint pv )
 {
    XMesaContext xmesa = (XMesaContext) ctx->DriverCtx;
-   GLubyte *color = ctx->VB->Color[pv];
-   XImage *img = xmesa->xm_buffer->backimage;
+   const GLubyte *color = ctx->VB->ColorPtr->data[pv];
+   XMesaImage *img = xmesa->xm_buffer->backimage;
    unsigned long pixel;
    PACK_TRUECOLOR( pixel, color[0], color[1], color[2] );
 
 #define INTERP_XY 1
 #define CLIP_HACK 1
-#define PLOT(X,Y) XPutPixel( img, X, FLIP(Y), pixel );
+#define PLOT(X,Y) XMesaPutPixel( img, X, FLIP(xmesa->xm_buffer, Y), pixel );
 
 #include "linetemp.h"
 }
@@ -217,12 +179,12 @@ static void flat_8A8B8G8R_line( GLcontext *ctx,
                                 GLuint vert0, GLuint vert1, GLuint pv )
 {
    XMesaContext xmesa = (XMesaContext) ctx->DriverCtx;
-   GLubyte *color = ctx->VB->Color[pv];
+   const GLubyte *color = ctx->VB->ColorPtr->data[pv];
    GLuint pixel = PACK_8B8G8R( color[0], color[1], color[2] );
 
 #define PIXEL_TYPE GLuint
 #define BYTES_PER_ROW (xmesa->xm_buffer->backimage->bytes_per_line)
-#define PIXEL_ADDRESS(X,Y) PIXELADDR4(X,Y)
+#define PIXEL_ADDRESS(X,Y) PIXELADDR4(xmesa->xm_buffer,X,Y)
 #define CLIP_HACK 1
 #define PLOT(X,Y) *pixelPtr = pixel;
 
@@ -237,12 +199,12 @@ static void flat_8R8G8B_line( GLcontext *ctx,
                               GLuint vert0, GLuint vert1, GLuint pv )
 {
    XMesaContext xmesa = (XMesaContext) ctx->DriverCtx;
-   GLubyte *color = ctx->VB->Color[pv];
+   const GLubyte *color = ctx->VB->ColorPtr->data[pv];
    GLuint pixel = PACK_8R8G8B( color[0], color[1], color[2] );
 
 #define PIXEL_TYPE GLuint
 #define BYTES_PER_ROW (xmesa->xm_buffer->backimage->bytes_per_line)
-#define PIXEL_ADDRESS(X,Y) PIXELADDR4(X,Y)
+#define PIXEL_ADDRESS(X,Y) PIXELADDR4(xmesa->xm_buffer,X,Y)
 #define CLIP_HACK 1
 #define PLOT(X,Y) *pixelPtr = pixel;
 
@@ -250,6 +212,29 @@ static void flat_8R8G8B_line( GLcontext *ctx,
 }
 
 
+/*
+ * Draw a flat-shaded, PF_8R8G8B24 line into an XImage.
+ */
+static void flat_8R8G8B24_line( GLcontext *ctx,
+                              GLuint vert0, GLuint vert1, GLuint pv )
+{
+   XMesaContext xmesa = (XMesaContext) ctx->DriverCtx;
+   const GLubyte *color = ctx->VB->ColorPtr->data[pv];
+
+#define PIXEL_TYPE bgr_t
+#define BYTES_PER_ROW (xmesa->xm_buffer->backimage->bytes_per_line)
+#define PIXEL_ADDRESS(X,Y) PIXELADDR3(xmesa->xm_buffer,X,Y)
+#define CLIP_HACK 1
+#define PLOT(X,Y) {			\
+      pixelPtr->r = color[RCOMP];	\
+      pixelPtr->g = color[GCOMP];	\
+      pixelPtr->b = color[BCOMP];	\
+}
+
+#include "linetemp.h"
+}
+
+
 /*
  * Draw a flat-shaded, PF_5R6G5B line into an XImage.
  */
@@ -257,12 +242,12 @@ static void flat_5R6G5B_line( GLcontext *ctx,
                               GLuint vert0, GLuint vert1, GLuint pv )
 {
    XMesaContext xmesa = (XMesaContext) ctx->DriverCtx;
-   GLubyte *color = ctx->VB->Color[pv];
+   const GLubyte *color = ctx->VB->ColorPtr->data[pv];
    GLushort pixel = PACK_5R6G5B( color[0], color[1], color[2] );
 
 #define PIXEL_TYPE GLushort
 #define BYTES_PER_ROW (xmesa->xm_buffer->backimage->bytes_per_line)
-#define PIXEL_ADDRESS(X,Y) PIXELADDR2(X,Y)
+#define PIXEL_ADDRESS(X,Y) PIXELADDR2(xmesa->xm_buffer,X,Y)
 #define CLIP_HACK 1
 #define PLOT(X,Y) *pixelPtr = pixel;
 
@@ -270,6 +255,25 @@ static void flat_5R6G5B_line( GLcontext *ctx,
 }
 
 
+/*
+ * Draw a flat-shaded, PF_DITHER_5R6G5B line into an XImage.
+ */
+static void flat_DITHER_5R6G5B_line( GLcontext *ctx,
+                                     GLuint vert0, GLuint vert1, GLuint pv )
+{
+   XMesaContext xmesa = (XMesaContext) ctx->DriverCtx;
+   const GLubyte *color = ctx->VB->ColorPtr->data[pv];
+
+#define PIXEL_TYPE GLushort
+#define BYTES_PER_ROW (xmesa->xm_buffer->backimage->bytes_per_line)
+#define PIXEL_ADDRESS(X,Y) PIXELADDR2(xmesa->xm_buffer,X,Y)
+#define CLIP_HACK 1
+#define PLOT(X,Y) PACK_TRUEDITHER( *pixelPtr, X, Y, color[0], color[1], color[2] );
+
+#include "linetemp.h"
+}
+
+
 
 /*
  * Draw a flat-shaded, PF_DITHER 8-bit line into an XImage.
@@ -278,14 +282,14 @@ static void flat_DITHER8_line( GLcontext *ctx,
                                GLuint vert0, GLuint vert1, GLuint pv )
 {
    XMesaContext xmesa = (XMesaContext) ctx->DriverCtx;
-   GLubyte *color = ctx->VB->Color[pv];
+   const GLubyte *color = ctx->VB->ColorPtr->data[pv];
    GLint r = color[0], g = color[1], b = color[2];
    DITHER_SETUP;
 
 #define INTERP_XY 1
 #define PIXEL_TYPE GLubyte
 #define BYTES_PER_ROW (xmesa->xm_buffer->backimage->bytes_per_line)
-#define PIXEL_ADDRESS(X,Y) PIXELADDR1(X,Y)
+#define PIXEL_ADDRESS(X,Y) PIXELADDR1(xmesa->xm_buffer,X,Y)
 #define CLIP_HACK 1
 #define PLOT(X,Y) *pixelPtr = DITHER(X,Y,r,g,b);
 
@@ -300,14 +304,14 @@ static void flat_LOOKUP8_line( GLcontext *ctx,
                                GLuint vert0, GLuint vert1, GLuint pv )
 {
    XMesaContext xmesa = (XMesaContext) ctx->DriverCtx;
-   GLubyte *color = ctx->VB->Color[pv];
+   const GLubyte *color = ctx->VB->ColorPtr->data[pv];
    GLubyte pixel;
    LOOKUP_SETUP;
-   pixel = LOOKUP( color[0], color[1], color[2] );
+   pixel = (GLubyte) LOOKUP( color[0], color[1], color[2] );
 
 #define PIXEL_TYPE GLubyte
 #define BYTES_PER_ROW (xmesa->xm_buffer->backimage->bytes_per_line)
-#define PIXEL_ADDRESS(X,Y) PIXELADDR1(X,Y)
+#define PIXEL_ADDRESS(X,Y) PIXELADDR1(xmesa->xm_buffer,X,Y)
 #define CLIP_HACK 1
 #define PLOT(X,Y) *pixelPtr = pixel;
 
@@ -322,15 +326,15 @@ static void flat_HPCR_line( GLcontext *ctx,
                             GLuint vert0, GLuint vert1, GLuint pv )
 {
    XMesaContext xmesa = (XMesaContext) ctx->DriverCtx;
-   GLubyte *color = ctx->VB->Color[pv];
+   const GLubyte *color = ctx->VB->ColorPtr->data[pv];
    GLint r = color[0], g = color[1], b = color[2];
 
 #define INTERP_XY 1
 #define PIXEL_TYPE GLubyte
 #define BYTES_PER_ROW (xmesa->xm_buffer->backimage->bytes_per_line)
-#define PIXEL_ADDRESS(X,Y) PIXELADDR1(X,Y)
+#define PIXEL_ADDRESS(X,Y) PIXELADDR1(xmesa->xm_buffer,X,Y)
 #define CLIP_HACK 1
-#define PLOT(X,Y) *pixelPtr = DITHER_HPCR(X,Y,r,g,b);
+#define PLOT(X,Y) *pixelPtr = (GLubyte) DITHER_HPCR(X,Y,r,g,b);
 
 #include "linetemp.h"
 }
@@ -344,18 +348,19 @@ static void flat_TRUECOLOR_z_line( GLcontext *ctx,
                                    GLuint vert0, GLuint vert1, GLuint pv )
 {
    XMesaContext xmesa = (XMesaContext) ctx->DriverCtx;
-   GLubyte *color = ctx->VB->Color[pv];
-   XImage *img = xmesa->xm_buffer->backimage;
+   const GLubyte *color = ctx->VB->ColorPtr->data[pv];
+   XMesaImage *img = xmesa->xm_buffer->backimage;
    unsigned long pixel;
    PACK_TRUECOLOR( pixel, color[0], color[1], color[2] );
 
 #define INTERP_XY 1
 #define INTERP_Z 1
+#define DEPTH_TYPE DEFAULT_SOFTWARE_DEPTH_TYPE
 #define CLIP_HACK 1
-#define PLOT(X,Y)				\
-	if (Z < *zPtr) {			\
-	   *zPtr = Z;				\
-           XPutPixel( img, X, FLIP(Y), pixel );	\
+#define PLOT(X,Y)							\
+	if (Z < *zPtr) {						\
+	   *zPtr = Z;							\
+           XMesaPutPixel( img, X, FLIP(xmesa->xm_buffer, Y), pixel );	\
 	}
 
 #include "linetemp.h"
@@ -369,13 +374,14 @@ static void flat_8A8B8G8R_z_line( GLcontext *ctx,
                                   GLuint vert0, GLuint vert1, GLuint pv )
 {
    XMesaContext xmesa = (XMesaContext) ctx->DriverCtx;
-   GLubyte *color = ctx->VB->Color[pv];
+   const GLubyte *color = ctx->VB->ColorPtr->data[pv];
    GLuint pixel = PACK_8B8G8R( color[0], color[1], color[2] );
 
 #define INTERP_Z 1
+#define DEPTH_TYPE DEFAULT_SOFTWARE_DEPTH_TYPE
 #define PIXEL_TYPE GLuint
 #define BYTES_PER_ROW (xmesa->xm_buffer->backimage->bytes_per_line)
-#define PIXEL_ADDRESS(X,Y) PIXELADDR4(X,Y)
+#define PIXEL_ADDRESS(X,Y) PIXELADDR4(xmesa->xm_buffer,X,Y)
 #define CLIP_HACK 1
 #define PLOT(X,Y)		\
 	if (Z < *zPtr) {	\
@@ -394,13 +400,14 @@ static void flat_8R8G8B_z_line( GLcontext *ctx,
                                 GLuint vert0, GLuint vert1, GLuint pv )
 {
    XMesaContext xmesa = (XMesaContext) ctx->DriverCtx;
-   GLubyte *color = ctx->VB->Color[pv];
+   const GLubyte *color = ctx->VB->ColorPtr->data[pv];
    GLuint pixel = PACK_8R8G8B( color[0], color[1], color[2] );
 
 #define INTERP_Z 1
+#define DEPTH_TYPE DEFAULT_SOFTWARE_DEPTH_TYPE
 #define PIXEL_TYPE GLuint
 #define BYTES_PER_ROW (xmesa->xm_buffer->backimage->bytes_per_line)
-#define PIXEL_ADDRESS(X,Y) PIXELADDR4(X,Y)
+#define PIXEL_ADDRESS(X,Y) PIXELADDR4(xmesa->xm_buffer,X,Y)
 #define CLIP_HACK 1
 #define PLOT(X,Y)		\
 	if (Z < *zPtr) {	\
@@ -412,6 +419,33 @@ static void flat_8R8G8B_z_line( GLcontext *ctx,
 }
 
 
+/*
+ * Draw a flat-shaded, Z-less, PF_8R8G8B24 line into an XImage.
+ */
+static void flat_8R8G8B24_z_line( GLcontext *ctx,
+                                    GLuint vert0, GLuint vert1, GLuint pv )
+{
+   XMesaContext xmesa = (XMesaContext) ctx->DriverCtx;
+   const GLubyte *color = ctx->VB->ColorPtr->data[pv];
+
+#define INTERP_Z 1
+#define DEPTH_TYPE DEFAULT_SOFTWARE_DEPTH_TYPE
+#define PIXEL_TYPE bgr_t
+#define BYTES_PER_ROW (xmesa->xm_buffer->backimage->bytes_per_line)
+#define PIXEL_ADDRESS(X,Y) PIXELADDR3(xmesa->xm_buffer,X,Y)
+#define CLIP_HACK 1
+#define PLOT(X,Y)			\
+	if (Z < *zPtr) {		\
+	   *zPtr = Z;			\
+           pixelPtr->r = color[RCOMP];	\
+           pixelPtr->g = color[GCOMP];	\
+           pixelPtr->b = color[BCOMP];	\
+	}
+
+#include "linetemp.h"
+}
+
+
 /*
  * Draw a flat-shaded, Z-less, PF_5R6G5B line into an XImage.
  */
@@ -419,13 +453,14 @@ static void flat_5R6G5B_z_line( GLcontext *ctx,
                                 GLuint vert0, GLuint vert1, GLuint pv )
 {
    XMesaContext xmesa = (XMesaContext) ctx->DriverCtx;
-   GLubyte *color = ctx->VB->Color[pv];
+   const GLubyte *color = ctx->VB->ColorPtr->data[pv];
    GLushort pixel = PACK_5R6G5B( color[0], color[1], color[2] );
 
 #define INTERP_Z 1
+#define DEPTH_TYPE DEFAULT_SOFTWARE_DEPTH_TYPE
 #define PIXEL_TYPE GLushort
 #define BYTES_PER_ROW (xmesa->xm_buffer->backimage->bytes_per_line)
-#define PIXEL_ADDRESS(X,Y) PIXELADDR2(X,Y)
+#define PIXEL_ADDRESS(X,Y) PIXELADDR2(xmesa->xm_buffer,X,Y)
 #define CLIP_HACK 1
 #define PLOT(X,Y)		\
 	if (Z < *zPtr) {	\
@@ -436,6 +471,30 @@ static void flat_5R6G5B_z_line( GLcontext *ctx,
 }
 
 
+/*
+ * Draw a flat-shaded, Z-less, PF_DITHER_5R6G5B line into an XImage.
+ */
+static void flat_DITHER_5R6G5B_z_line( GLcontext *ctx,
+                                       GLuint vert0, GLuint vert1, GLuint pv )
+{
+   XMesaContext xmesa = (XMesaContext) ctx->DriverCtx;
+   const GLubyte *color = ctx->VB->ColorPtr->data[pv];
+
+#define INTERP_Z 1
+#define DEPTH_TYPE DEFAULT_SOFTWARE_DEPTH_TYPE
+#define PIXEL_TYPE GLushort
+#define BYTES_PER_ROW (xmesa->xm_buffer->backimage->bytes_per_line)
+#define PIXEL_ADDRESS(X,Y) PIXELADDR2(xmesa->xm_buffer,X,Y)
+#define CLIP_HACK 1
+#define PLOT(X,Y)		\
+	if (Z < *zPtr) {	\
+	   *zPtr = Z;		\
+	   PACK_TRUEDITHER(*pixelPtr, X, Y, color[0], color[1], color[2]); \
+	}
+#include "linetemp.h"
+}
+
+
 /*
  * Draw a flat-shaded, Z-less, PF_DITHER 8-bit line into an XImage.
  */
@@ -443,20 +502,21 @@ static void flat_DITHER8_z_line( GLcontext *ctx,
                                  GLuint vert0, GLuint vert1, GLuint pv )
 {
    XMesaContext xmesa = (XMesaContext) ctx->DriverCtx;
-   GLubyte *color = ctx->VB->Color[pv];
+   const GLubyte *color = ctx->VB->ColorPtr->data[pv];
    GLint r = color[0], g = color[1], b = color[2];
    DITHER_SETUP;
 
 #define INTERP_XY 1
 #define INTERP_Z 1
+#define DEPTH_TYPE DEFAULT_SOFTWARE_DEPTH_TYPE
 #define PIXEL_TYPE GLubyte
 #define BYTES_PER_ROW (xmesa->xm_buffer->backimage->bytes_per_line)
-#define PIXEL_ADDRESS(X,Y) PIXELADDR1(X,Y)
+#define PIXEL_ADDRESS(X,Y) PIXELADDR1(xmesa->xm_buffer,X,Y)
 #define CLIP_HACK 1
-#define PLOT(X,Y)				\
-	if (Z < *zPtr) {			\
-	   *zPtr = Z;				\
-	   *pixelPtr = DITHER( X, Y, r, g, b);	\
+#define PLOT(X,Y)						\
+	if (Z < *zPtr) {					\
+	   *zPtr = Z;						\
+	   *pixelPtr = (GLubyte) DITHER( X, Y, r, g, b);	\
 	}
 #include "linetemp.h"
 }
@@ -469,15 +529,16 @@ static void flat_LOOKUP8_z_line( GLcontext *ctx,
                                  GLuint vert0, GLuint vert1, GLuint pv )
 {
    XMesaContext xmesa = (XMesaContext) ctx->DriverCtx;
-   GLubyte *color = ctx->VB->Color[pv];
+   const GLubyte *color = ctx->VB->ColorPtr->data[pv];
    GLubyte pixel;
    LOOKUP_SETUP;
-   pixel = LOOKUP( color[0], color[1], color[2] );
+   pixel = (GLubyte) LOOKUP( color[0], color[1], color[2] );
 
 #define INTERP_Z 1
+#define DEPTH_TYPE DEFAULT_SOFTWARE_DEPTH_TYPE
 #define PIXEL_TYPE GLubyte
 #define BYTES_PER_ROW (xmesa->xm_buffer->backimage->bytes_per_line)
-#define PIXEL_ADDRESS(X,Y) PIXELADDR1(X,Y)
+#define PIXEL_ADDRESS(X,Y) PIXELADDR1(xmesa->xm_buffer,X,Y)
 #define CLIP_HACK 1
 #define PLOT(X,Y)		\
 	if (Z < *zPtr) {	\
@@ -496,19 +557,20 @@ static void flat_HPCR_z_line( GLcontext *ctx,
                               GLuint vert0, GLuint vert1, GLuint pv )
 {
    XMesaContext xmesa = (XMesaContext) ctx->DriverCtx;
-   GLubyte *color = ctx->VB->Color[pv];
+   const GLubyte *color = ctx->VB->ColorPtr->data[pv];
    GLint r = color[0], g = color[1], b = color[2];
 
 #define INTERP_XY 1
 #define INTERP_Z 1
+#define DEPTH_TYPE DEFAULT_SOFTWARE_DEPTH_TYPE
 #define PIXEL_TYPE GLubyte
 #define BYTES_PER_ROW (xmesa->xm_buffer->backimage->bytes_per_line)
-#define PIXEL_ADDRESS(X,Y) PIXELADDR1(X,Y)
+#define PIXEL_ADDRESS(X,Y) PIXELADDR1(xmesa->xm_buffer,X,Y)
 #define CLIP_HACK 1
-#define PLOT(X,Y)					\
-	if (Z < *zPtr) {				\
-	   *zPtr = Z;					\
-	   *pixelPtr = DITHER_HPCR( X, Y, r, g, b);	\
+#define PLOT(X,Y)						\
+	if (Z < *zPtr) {					\
+	   *zPtr = Z;						\
+	   *pixelPtr = (GLubyte) DITHER_HPCR( X, Y, r, g, b);	\
 	}
 
 #include "linetemp.h"
@@ -568,8 +630,10 @@ fprintf (stderr, " %d", dashes[i]);
 fprintf (stderr, "\n");
 #endif
 
-      XSetDashes( xmesa->display, xmesa->xm_buffer->gc1, offs, dashes, n_segments );
-      XSetDashes( xmesa->display, xmesa->xm_buffer->gc2, offs, dashes, n_segments );
+      XMesaSetDashes( xmesa->display, xmesa->xm_buffer->gc1,
+		      offs, dashes, n_segments );
+      XMesaSetDashes( xmesa->display, xmesa->xm_buffer->gc2,
+		      offs, dashes, n_segments );
 
       line_style = LineOnOffDash;
    }
@@ -585,12 +649,12 @@ fprintf (stderr, "\n");
    }
 
    /*** Set GC attributes ***/
-   XSetLineAttributes( xmesa->display, xmesa->xm_buffer->gc1,
-                       line_width, line_style, CapButt, JoinBevel);
-   XSetLineAttributes( xmesa->display, xmesa->xm_buffer->gc2,
-                       line_width, line_style, CapButt, JoinBevel);
-   XSetFillStyle( xmesa->display, xmesa->xm_buffer->gc1, FillSolid );
-   XSetFillStyle( xmesa->display, xmesa->xm_buffer->gc2, FillSolid );
+   XMesaSetLineAttributes( xmesa->display, xmesa->xm_buffer->gc1,
+			   line_width, line_style, CapButt, JoinBevel);
+   XMesaSetLineAttributes( xmesa->display, xmesa->xm_buffer->gc2,
+			   line_width, line_style, CapButt, JoinBevel);
+   XMesaSetFillStyle( xmesa->display, xmesa->xm_buffer->gc1, FillSolid );
+   XMesaSetFillStyle( xmesa->display, xmesa->xm_buffer->gc2, FillSolid );
 }
 
 
@@ -602,7 +666,10 @@ fprintf (stderr, "\n");
 line_func xmesa_get_line_func( GLcontext *ctx )
 {
    XMesaContext xmesa = (XMesaContext) ctx->DriverCtx;
-   int depth = xmesa->xm_visual->visinfo->depth;
+   int depth = GET_VISUAL_DEPTH(xmesa->xm_visual);
+
+   (void) DitherValues;  /* silence unused var warning */
+   (void) kernel1;  /* silence unused var warning */
 
    if (ctx->Line.SmoothFlag)              return NULL;
    if (ctx->Texture.Enabled)              return NULL;
@@ -614,6 +681,7 @@ line_func xmesa_get_line_func( GLcontext *ctx )
        && ctx->RasterMask==DEPTH_BIT
        && ctx->Depth.Func==GL_LESS
        && ctx->Depth.Mask==GL_TRUE
+       && ctx->Visual->DepthBits == DEFAULT_SOFTWARE_DEPTH_BITS
        && ctx->Line.Width==1.0F) {
       switch (xmesa->pixelformat) {
          case PF_TRUECOLOR:
@@ -622,8 +690,12 @@ line_func xmesa_get_line_func( GLcontext *ctx )
             return flat_8A8B8G8R_z_line;
          case PF_8R8G8B:
             return flat_8R8G8B_z_line;
+         case PF_8R8G8B24:
+            return flat_8R8G8B24_z_line;
          case PF_5R6G5B:
             return flat_5R6G5B_z_line;
+         case PF_DITHER_5R6G5B:
+            return flat_DITHER_5R6G5B_z_line;
          case PF_DITHER:
             return (depth==8) ? flat_DITHER8_z_line : NULL;
          case PF_LOOKUP:
@@ -644,8 +716,12 @@ line_func xmesa_get_line_func( GLcontext *ctx )
             return flat_8A8B8G8R_line;
          case PF_8R8G8B:
             return flat_8R8G8B_line;
+         case PF_8R8G8B24:
+            return flat_8R8G8B24_line;
          case PF_5R6G5B:
             return flat_5R6G5B_line;
+         case PF_DITHER_5R6G5B:
+            return flat_DITHER_5R6G5B_line;
          case PF_DITHER:
             return (depth==8) ? flat_DITHER8_line : NULL;
          case PF_LOOKUP:
@@ -656,9 +732,15 @@ line_func xmesa_get_line_func( GLcontext *ctx )
 	    return NULL;
       }
    }
+#if 0
+   /* XXX have to disable this because X's rasterization rules don't match
+    * software Mesa's.  This causes the linehv.c conformance test to fail.
+    * In the future, we might provide a config option to enable this.
+    */
    if (xmesa->xm_buffer->buffer!=XIMAGE && ctx->RasterMask==0) {
       setup_x_line_options( ctx );
       return flat_pixmap_line;
    }
+#endif
    return NULL;
 }
