@@ -1,8 +1,8 @@
-/* $Id: light.c,v 1.8.2.10 2000/07/17 12:49:56 keithw Exp $ */
+/* $Id: light.c,v 1.16 2000/07/18 16:55:56 brianp Exp $ */
 
 /*
  * Mesa 3-D graphics library
- * Version:  3.2
+ * Version:  3.3
  * 
  * Copyright (C) 1999-2000  Brian Paul   All Rights Reserved.
  * 
@@ -25,27 +25,16 @@
  */
 
 
-
-
-
 #ifdef PC_HEADER
 #include "all.h"
 #else
-#include <float.h>
-#ifndef XFree86Server
-#include <assert.h>
-#include <float.h>
-#include <math.h>
-#include <stdlib.h>
-#include <stdio.h>
-#else
-#include "GL/xf86glx.h"
-#endif
+#include "glheader.h"
 #include "context.h"
 #include "enums.h"
 #include "light.h"
 #include "macros.h"
 #include "matrix.h"
+#include "mem.h"
 #include "mmath.h"
 #include "simple_list.h"
 #include "types.h"
@@ -54,9 +43,16 @@
 #endif
 
 
+/* XXX this is a bit of a hack needed for compilation within XFree86 */
+#ifndef FLT_MIN
+#define FLT_MIN 1e-37
+#endif
 
-void gl_ShadeModel( GLcontext *ctx, GLenum mode )
+
+void
+_mesa_ShadeModel( GLenum mode )
 {
+   GET_CURRENT_CONTEXT(ctx);
    ASSERT_OUTSIDE_BEGIN_END_AND_FLUSH(ctx, "glShadeModel");
 
    if (MESA_VERBOSE & VERBOSE_API)
@@ -64,12 +60,12 @@ void gl_ShadeModel( GLcontext *ctx, GLenum mode )
 
    if (mode == GL_FLAT || mode == GL_SMOOTH) {
       if (ctx->Light.ShadeModel != mode) {
-	 ctx->Light.ShadeModel = mode;
+         ctx->Light.ShadeModel = mode;
          if (ctx->Light.ShadeModel == GL_FLAT)
-            ctx->TriangleCaps |= DD_FLATSHADE;
+            SET_BITS(ctx->TriangleCaps, DD_FLATSHADE);
          else
-            ctx->TriangleCaps &= ~DD_FLATSHADE;
-	 ctx->NewState |= NEW_RASTER_OPS;
+            CLEAR_BITS(ctx->TriangleCaps, DD_FLATSHADE);
+         ctx->NewState |= NEW_RASTER_OPS;
          if (ctx->Driver.ShadeModel) 
             (*ctx->Driver.ShadeModel)( ctx, mode );
       }
@@ -81,19 +77,25 @@ void gl_ShadeModel( GLcontext *ctx, GLenum mode )
 
 
 
-void gl_Lightfv( GLcontext *ctx,
-                 GLenum light, GLenum pname, const GLfloat *params,
-                 GLint nparams )
+void
+_mesa_Lightf( GLenum light, GLenum pname, GLfloat param )
 {
-   GLint l;
+   _mesa_Lightfv( light, pname, &param );
+}
 
-   (void) nparams;
+
+void
+_mesa_Lightfv( GLenum light, GLenum pname, const GLfloat *params )
+{
+   GET_CURRENT_CONTEXT(ctx);
+   GLint l;
+   GLint nParams;
 
    ASSERT_OUTSIDE_BEGIN_END_AND_FLUSH(ctx, "glLight");
 
    l = (GLint) (light - GL_LIGHT0);
 
-   if (l<0 || l>=MAX_LIGHTS) {
+   if (l < 0 || l >= MAX_LIGHTS) {
       gl_error( ctx, GL_INVALID_ENUM, "glLight" );
       return;
    }
@@ -101,18 +103,22 @@ void gl_Lightfv( GLcontext *ctx,
    switch (pname) {
       case GL_AMBIENT:
          COPY_4V( ctx->Light.Light[l].Ambient, params );
+         nParams = 4;
          break;
       case GL_DIFFUSE:
          COPY_4V( ctx->Light.Light[l].Diffuse, params );
+         nParams = 4;
          break;
       case GL_SPECULAR:
          COPY_4V( ctx->Light.Light[l].Specular, params );
+         nParams = 4;
          break;
       case GL_POSITION:
 	 /* transform position by ModelView matrix */
 	 TRANSFORM_POINT( ctx->Light.Light[l].EyePosition, 
 			  ctx->ModelView.m,
                           params );
+         nParams = 4;
          break;
       case GL_SPOT_DIRECTION:
 	 /* transform direction by inverse modelview */
@@ -122,6 +128,7 @@ void gl_Lightfv( GLcontext *ctx,
 	 TRANSFORM_NORMAL( ctx->Light.Light[l].EyeDirection,
 			   params,
 			   ctx->ModelView.inv );
+         nParams = 3;
          break;
       case GL_SPOT_EXPONENT:
          if (params[0]<0.0 || params[0]>128.0) {
@@ -132,6 +139,7 @@ void gl_Lightfv( GLcontext *ctx,
             ctx->Light.Light[l].SpotExponent = params[0];
             gl_compute_spot_exp_table( &ctx->Light.Light[l] );
          }
+         nParams = 1;
          break;
       case GL_SPOT_CUTOFF:
          if ((params[0]<0.0 || params[0]>90.0) && params[0]!=180.0) {
@@ -142,6 +150,7 @@ void gl_Lightfv( GLcontext *ctx,
          ctx->Light.Light[l].CosCutoff = cos(params[0]*DEG2RAD);
          if (ctx->Light.Light[l].CosCutoff < 0) 
 	    ctx->Light.Light[l].CosCutoff = 0;
+         nParams = 1;
          break;
       case GL_CONSTANT_ATTENUATION:
          if (params[0]<0.0) {
@@ -149,6 +158,7 @@ void gl_Lightfv( GLcontext *ctx,
             return;
          }
          ctx->Light.Light[l].ConstantAttenuation = params[0];
+         nParams = 1;
          break;
       case GL_LINEAR_ATTENUATION:
          if (params[0]<0.0) {
@@ -156,6 +166,7 @@ void gl_Lightfv( GLcontext *ctx,
             return;
          }
          ctx->Light.Light[l].LinearAttenuation = params[0];
+         nParams = 1;
          break;
       case GL_QUADRATIC_ATTENUATION:
          if (params[0]<0.0) {
@@ -163,23 +174,73 @@ void gl_Lightfv( GLcontext *ctx,
             return;
          }
          ctx->Light.Light[l].QuadraticAttenuation = params[0];
+         nParams = 1;
          break;
       default:
          gl_error( ctx, GL_INVALID_ENUM, "glLight" );
-         break;
+         return;
    }
 
    if (ctx->Driver.Lightfv)
-      ctx->Driver.Lightfv( ctx, light, pname, params, nparams );
+      ctx->Driver.Lightfv( ctx, light, pname, params, nParams );
 
    ctx->NewState |= NEW_LIGHTING;
 }
 
 
-
-void gl_GetLightfv( GLcontext *ctx,
-                    GLenum light, GLenum pname, GLfloat *params )
+void
+_mesa_Lighti( GLenum light, GLenum pname, GLint param )
 {
+   _mesa_Lightiv( light, pname, &param );
+}
+
+
+void
+_mesa_Lightiv( GLenum light, GLenum pname, const GLint *params )
+{
+   GLfloat fparam[4];
+
+   switch (pname) {
+      case GL_AMBIENT:
+      case GL_DIFFUSE:
+      case GL_SPECULAR:
+         fparam[0] = INT_TO_FLOAT( params[0] );
+         fparam[1] = INT_TO_FLOAT( params[1] );
+         fparam[2] = INT_TO_FLOAT( params[2] );
+         fparam[3] = INT_TO_FLOAT( params[3] );
+         break;
+      case GL_POSITION:
+         fparam[0] = (GLfloat) params[0];
+         fparam[1] = (GLfloat) params[1];
+         fparam[2] = (GLfloat) params[2];
+         fparam[3] = (GLfloat) params[3];
+         break;
+      case GL_SPOT_DIRECTION:
+         fparam[0] = (GLfloat) params[0];
+         fparam[1] = (GLfloat) params[1];
+         fparam[2] = (GLfloat) params[2];
+         break;
+      case GL_SPOT_EXPONENT:
+      case GL_SPOT_CUTOFF:
+      case GL_CONSTANT_ATTENUATION:
+      case GL_LINEAR_ATTENUATION:
+      case GL_QUADRATIC_ATTENUATION:
+         fparam[0] = (GLfloat) params[0];
+         break;
+      default:
+         /* error will be caught later in gl_Lightfv */
+         ;
+   }
+
+   _mesa_Lightfv( light, pname, fparam );
+}
+
+
+
+void
+_mesa_GetLightfv( GLenum light, GLenum pname, GLfloat *params )
+{
+   GET_CURRENT_CONTEXT(ctx);
    GLint l = (GLint) (light - GL_LIGHT0);
 
    ASSERT_OUTSIDE_BEGIN_END_AND_FLUSH(ctx, "glGetLight");
@@ -228,8 +289,10 @@ void gl_GetLightfv( GLcontext *ctx,
 
 
 
-void gl_GetLightiv( GLcontext *ctx, GLenum light, GLenum pname, GLint *params )
+void
+_mesa_GetLightiv( GLenum light, GLenum pname, GLint *params )
 {
+   GET_CURRENT_CONTEXT(ctx);
    GLint l = (GLint) (light - GL_LIGHT0);
 
    ASSERT_OUTSIDE_BEGIN_END_AND_FLUSH(ctx, "glGetLight");
@@ -297,9 +360,11 @@ void gl_GetLightiv( GLcontext *ctx, GLenum light, GLenum pname, GLint *params )
 /**********************************************************************/
 
 
-void gl_LightModelfv( GLcontext *ctx, GLenum pname, const GLfloat *params )
+void
+_mesa_LightModelfv( GLenum pname, const GLfloat *params )
 {
-   ASSERT_OUTSIDE_BEGIN_END_AND_FLUSH(ctx, "glLightModel");
+   GET_CURRENT_CONTEXT(ctx);
+   ASSERT_OUTSIDE_BEGIN_END_AND_FLUSH(ctx, "glLightModelfv");
 
    switch (pname) {
       case GL_LIGHT_MODEL_AMBIENT:
@@ -320,11 +385,11 @@ void gl_LightModelfv( GLcontext *ctx, GLenum pname, const GLfloat *params )
       case GL_LIGHT_MODEL_COLOR_CONTROL:
          if (params[0] == (GLfloat) GL_SINGLE_COLOR) {
             ctx->Light.Model.ColorControl = GL_SINGLE_COLOR;
-            ctx->TriangleCaps &= ~DD_SEPERATE_SPECULAR;
+            CLEAR_BITS(ctx->TriangleCaps, DD_SEPERATE_SPECULAR);
          }
          else if (params[0] == (GLfloat) GL_SEPARATE_SPECULAR_COLOR) {
             ctx->Light.Model.ColorControl = GL_SEPARATE_SPECULAR_COLOR;
-	    ctx->TriangleCaps |= DD_SEPERATE_SPECULAR;
+	    SET_BITS(ctx->TriangleCaps, DD_SEPERATE_SPECULAR);
 	 }
          else {
             gl_error( ctx, GL_INVALID_ENUM, "glLightModel(param)" );
@@ -343,6 +408,46 @@ void gl_LightModelfv( GLcontext *ctx, GLenum pname, const GLfloat *params )
 }
 
 
+void
+_mesa_LightModeliv( GLenum pname, const GLint *params )
+{
+   GLfloat fparam[4];
+   GET_CURRENT_CONTEXT(ctx);
+   ASSERT_OUTSIDE_BEGIN_END_AND_FLUSH(ctx, "glLightModeliv");
+
+   switch (pname) {
+      case GL_LIGHT_MODEL_AMBIENT:
+         fparam[0] = INT_TO_FLOAT( params[0] );
+         fparam[1] = INT_TO_FLOAT( params[1] );
+         fparam[2] = INT_TO_FLOAT( params[2] );
+         fparam[3] = INT_TO_FLOAT( params[3] );
+         break;
+      case GL_LIGHT_MODEL_LOCAL_VIEWER:
+      case GL_LIGHT_MODEL_TWO_SIDE:
+      case GL_LIGHT_MODEL_COLOR_CONTROL:
+         fparam[0] = (GLfloat) params[0];
+         break;
+      default:
+         /* Error will be caught later in gl_LightModelfv */
+         ;
+   }
+   _mesa_LightModelfv( pname, fparam );
+}
+
+
+void
+_mesa_LightModeli( GLenum pname, GLint param )
+{
+   _mesa_LightModeliv( pname, &param );
+}
+
+
+void
+_mesa_LightModelf( GLenum pname, GLfloat param )
+{
+   _mesa_LightModelfv( pname, &param );
+}
+
 
 
 /********** MATERIAL **********/
@@ -408,9 +513,6 @@ GLuint gl_material_bitmask( GLcontext *ctx, GLenum face, GLenum pname,
 
 
 
-
-
-
 /*
  * Check if the global material has to be updated with info that was
  * associated with a vertex via glMaterial.
@@ -710,8 +812,11 @@ void gl_update_color_material( GLcontext *ctx,
 
 
 
-void gl_ColorMaterial( GLcontext *ctx, GLenum face, GLenum mode )
+
+void
+_mesa_ColorMaterial( GLenum face, GLenum mode )
 {
+   GET_CURRENT_CONTEXT(ctx);
    GLuint bitmask;
    GLuint legal = (FRONT_EMISSION_BIT | BACK_EMISSION_BIT |
 		   FRONT_SPECULAR_BIT | BACK_SPECULAR_BIT |
@@ -739,12 +844,21 @@ void gl_ColorMaterial( GLcontext *ctx, GLenum face, GLenum mode )
 
 
 
+
+void
+_mesa_Materialf( GLenum face, GLenum pname, GLfloat param )
+{
+   _mesa_Materialfv( face, pname, &param );
+}
+
+
 /* KW:  This is now called directly (ie by name) from the glMaterial* 
  *      API functions.
  */
-void gl_Materialfv( GLcontext *ctx,
-                    GLenum face, GLenum pname, const GLfloat *params )
+void
+_mesa_Materialfv( GLenum face, GLenum pname, const GLfloat *params )
 {
+   GET_CURRENT_CONTEXT(ctx);
    struct immediate *IM;
    struct gl_material *mat;
    GLuint bitmask;
@@ -770,6 +884,7 @@ void gl_Materialfv( GLcontext *ctx,
       IM->MaterialMask[count] = 0;      
    }
 
+
    IM->MaterialMask[count] |= bitmask;
    mat = IM->Material[count];
 
@@ -818,11 +933,48 @@ void gl_Materialfv( GLcontext *ctx,
 }
 
 
-
-
-void gl_GetMaterialfv( GLcontext *ctx,
-                       GLenum face, GLenum pname, GLfloat *params )
+void
+_mesa_Materiali(GLenum face, GLenum pname, GLint param )
 {
+   _mesa_Materialiv(face, pname, &param);
+}
+
+
+void
+_mesa_Materialiv(GLenum face, GLenum pname, const GLint *params )
+{
+   GLfloat fparam[4];
+   switch (pname) {
+      case GL_AMBIENT:
+      case GL_DIFFUSE:
+      case GL_SPECULAR:
+      case GL_EMISSION:
+      case GL_AMBIENT_AND_DIFFUSE:
+         fparam[0] = INT_TO_FLOAT( params[0] );
+         fparam[1] = INT_TO_FLOAT( params[1] );
+         fparam[2] = INT_TO_FLOAT( params[2] );
+         fparam[3] = INT_TO_FLOAT( params[3] );
+         break;
+      case GL_SHININESS:
+         fparam[0] = (GLfloat) params[0];
+         break;
+      case GL_COLOR_INDEXES:
+         fparam[0] = (GLfloat) params[0];
+         fparam[1] = (GLfloat) params[1];
+         fparam[2] = (GLfloat) params[2];
+         break;
+      default:
+         /* Error will be caught later in gl_Materialfv */
+         ;
+   }
+   _mesa_Materialfv(face, pname, fparam);
+}
+
+
+void
+_mesa_GetMaterialfv( GLenum face, GLenum pname, GLfloat *params )
+{
+   GET_CURRENT_CONTEXT(ctx);
    GLuint f;
 
    ASSERT_OUTSIDE_BEGIN_END_AND_FLUSH(ctx, "glGetMaterialfv");
@@ -865,9 +1017,10 @@ void gl_GetMaterialfv( GLcontext *ctx,
 
 
 
-void gl_GetMaterialiv( GLcontext *ctx,
-                       GLenum face, GLenum pname, GLint *params )
+void
+_mesa_GetMaterialiv( GLenum face, GLenum pname, GLint *params )
 {
+   GET_CURRENT_CONTEXT(ctx);
    GLuint f;
 
    ASSERT_OUTSIDE_BEGIN_END_AND_FLUSH(ctx, "glGetMaterialiv");
@@ -966,26 +1119,27 @@ void gl_GetMaterialiv( GLcontext *ctx,
  * Whenever the spotlight exponent for a light changes we must call
  * this function to recompute the exponent lookup table.
  */
-void gl_compute_spot_exp_table( struct gl_light *l )
+void
+gl_compute_spot_exp_table( struct gl_light *l )
 {
-   int i;
-   double exponent = l->SpotExponent;
-   double tmp = 0;
-   int clamp = 0;
+   GLint i;
+   GLdouble exponent = l->SpotExponent;
+   GLdouble tmp = 0;
+   GLint clamp = 0;
 
    l->SpotExpTable[0][0] = 0.0;
 
-   for (i=EXP_TABLE_SIZE-1;i>0;i--) {
+   for (i = EXP_TABLE_SIZE - 1; i > 0 ;i--) {
       if (clamp == 0) {
-         tmp = pow(i/(double)(EXP_TABLE_SIZE-1), exponent);
-         if (tmp < FLT_MIN*100.0) {
+         tmp = pow(i / (GLdouble) (EXP_TABLE_SIZE - 1), exponent);
+         if (tmp < FLT_MIN * 100.0) {
             tmp = 0.0;
             clamp = 1;
          }
       }
       l->SpotExpTable[i][0] = tmp;
    }
-   for (i=0;i<EXP_TABLE_SIZE-1;i++) {
+   for (i = 0; i < EXP_TABLE_SIZE - 1; i++) {
       l->SpotExpTable[i][1] = l->SpotExpTable[i+1][0] - l->SpotExpTable[i][0];
    }
    l->SpotExpTable[EXP_TABLE_SIZE-1][1] = 0.0;
@@ -998,50 +1152,63 @@ void gl_compute_spot_exp_table( struct gl_light *l )
  * lighting, and the cost of doing it early may be partially offset
  * by keeping a MRU cache of shine tables for various shine values.
  */
-static void compute_shine_table( struct gl_shine_tab *tab, GLfloat shininess )
+static void
+compute_shine_table( struct gl_shine_tab *tab, GLfloat shininess )
 {
-   int i;
+   GLint i;
    GLfloat *m = tab->tab;
 
-   for (i = 0 ; i < SHINE_TABLE_SIZE ; i++) {
-      double dp = i/(GLfloat)(SHINE_TABLE_SIZE-1);
-      double t = pow( dp, shininess );
-      if (t < 1e-20) t = 0;
-      m[i] = (GLfloat) t;
-   }      
+   m[0] = 0.0;
+   if (shininess == 0.0) {
+      for (i = 1 ; i <= SHINE_TABLE_SIZE ; i++)
+	 m[i] = 1.0;
+   }
+   else {
+      for (i = 1 ; i < SHINE_TABLE_SIZE ; i++) {
+	 GLdouble t = pow(i / (GLfloat) (SHINE_TABLE_SIZE - 1), shininess);
+	 if (t > 1e-20)
+            m[i] = t;
+         else
+            m[i] = 0.0;
+      }      
+      m[SHINE_TABLE_SIZE] = 1.0;
+   }
 
    tab->shininess = shininess;
 }
 
 
-void gl_compute_shine_table( GLcontext *ctx, GLuint i, GLfloat shininess )
+void
+gl_compute_shine_table( GLcontext *ctx, GLuint i, GLfloat shininess )
 {
+#define DISTSQR(a,b) ((a-b)*(a-b))
    struct gl_shine_tab *list = ctx->ShineTabList;
    struct gl_shine_tab *s;
 
    foreach(s, list) 
-      if ( s->shininess == shininess ) 
+      if ( DISTSQR(s->shininess, shininess) < 1e-4 ) 
 	 break;
 
-   if (s == list) 
-   {
+   if (s == list) {
       foreach(s, list) 
-	 if (s->refcount == 0) break;
+	 if (s->refcount == 0)
+            break;
 
       compute_shine_table( s, shininess );
    }
 
-/*     reset_shine_table( s, shininess ); */
    ctx->ShineTable[i]->refcount--;
    ctx->ShineTable[i] = s;
    move_to_tail( list, s );
    s->refcount++;
+#undef DISTSQR
 }
 
 
 
-
-void gl_reinit_light_attrib( GLcontext *ctx, struct gl_light_attrib *l )
+#if 0
+static void
+gl_reinit_light_attrib( GLcontext *ctx, struct gl_light_attrib *l )
 {
    GLuint i;
 
@@ -1061,7 +1228,7 @@ void gl_reinit_light_attrib( GLcontext *ctx, struct gl_light_attrib *l )
 	 insert_at_tail( &l->EnabledList, &l->Light[i] );
    }
 }
-
+#endif
 
 
 /*
@@ -1070,7 +1237,8 @@ void gl_reinit_light_attrib( GLcontext *ctx, struct gl_light_attrib *l )
  * Also, precompute some lighting values such as the products of light
  * source and material ambient, diffuse and specular coefficients.
  */
-void gl_update_lighting( GLcontext *ctx )
+void
+gl_update_lighting( GLcontext *ctx )
 {
    struct gl_light *light;
 
@@ -1100,13 +1268,12 @@ void gl_update_lighting( GLcontext *ctx )
 
    /* Precompute some shading values.
     */
-   if (ctx->Visual->RGBAflag) 
-   {
+   if (ctx->Visual->RGBAflag) {
       GLuint sides = ((ctx->TriangleCaps & DD_TRI_LIGHT_TWOSIDE) ? 2 : 1);
       GLuint side;
       for (side=0; side < sides; side++) {
 	 struct gl_material *mat = &ctx->Light.Material[side];
-	 
+
 	 COPY_3V(ctx->Light.BaseColor[side], mat->Emission);
 	 ACC_SCALE_3V(ctx->Light.BaseColor[side], 
 		      ctx->Light.Model.Ambient,
@@ -1118,11 +1285,10 @@ void gl_update_lighting( GLcontext *ctx )
       
       foreach (light, &ctx->Light.EnabledList) {	 
 	 for (side=0; side< sides; side++) {
-	    struct gl_material *mat = &ctx->Light.Material[side];
+	    const struct gl_material *mat = &ctx->Light.Material[side];
 	    SCALE_3V( light->MatDiffuse[side], light->Diffuse, mat->Diffuse );
 	    SCALE_3V( light->MatAmbient[side], light->Ambient, mat->Ambient );
-	    if (light->Flags & LIGHT_SPECULAR)
-	    {
+	    if (light->Flags & LIGHT_SPECULAR) {
 	       SCALE_3V( light->MatSpecular[side], light->Specular,
 			 mat->Specular);
 	       light->IsMatSpecular[side] = 
@@ -1133,10 +1299,8 @@ void gl_update_lighting( GLcontext *ctx )
 	 }
       }
    } 
-   else
-   {
-      static GLfloat ci[3] = { .30, .59, .11 };
-
+   else {
+      static const GLfloat ci[3] = { .30, .59, .11 };
       foreach(light, &ctx->Light.EnabledList) {
 	 light->dli = DOT3(ci, light->Diffuse);
 	 light->sli = DOT3(ci, light->Specular);
@@ -1149,7 +1313,8 @@ void gl_update_lighting( GLcontext *ctx )
 /* Need to seriously restrict the circumstances under which these
  * calc's are performed.
  */
-void gl_compute_light_positions( GLcontext *ctx )
+void
+gl_compute_light_positions( GLcontext *ctx )
 {
    struct gl_light *light;
    
@@ -1221,8 +1386,8 @@ void gl_compute_light_positions( GLcontext *ctx )
 }
 
 
-
-void gl_update_normal_transform( GLcontext *ctx )
+void
+gl_update_normal_transform( GLcontext *ctx )
 {
    GLuint new_flag = 0;
    normal_func *last = ctx->NormalTransform;
@@ -1243,20 +1408,18 @@ void gl_update_normal_transform( GLcontext *ctx )
 	 new_flag = ctx->NewState & NEW_MODELVIEW;
 	 ctx->vb_rescale_factor = ctx->rescale_factor;
 	       
-	 if (ctx->Transform.Normalize) 
-	 {
+	 if (ctx->Transform.Normalize) {
 	    ctx->NormalTransform = gl_normal_tab[transform | NORM_NORMALIZE];
 	 } 
 	 else if (ctx->Transform.RescaleNormals &&
-		  ctx->rescale_factor != 1.0)
-	 {
+		  ctx->rescale_factor != 1.0) {
 	    ctx->NormalTransform = gl_normal_tab[transform | NORM_RESCALE];
 	 }
-	 else 
-	 {
+	 else {
 	    ctx->NormalTransform = gl_normal_tab[transform];
 	 }
-      } else {
+      }
+      else {
 	 ctx->NormalTransform = 0;
       }
    }
@@ -1264,20 +1427,18 @@ void gl_update_normal_transform( GLcontext *ctx )
       if (ctx->NeedNormals) {
 	 ctx->vb_rescale_factor = 1.0/ctx->rescale_factor;
 
-	 if (ctx->Transform.Normalize) 
-	 {
+	 if (ctx->Transform.Normalize) {
 	    ctx->NormalTransform = gl_normal_tab[NORM_NORMALIZE];
 	 }
 	 else if (!ctx->Transform.RescaleNormals &&
-		  ctx->rescale_factor != 1.0)
-	 {
+		  ctx->rescale_factor != 1.0) {
 	    ctx->NormalTransform = gl_normal_tab[NORM_RESCALE];
 	 }
-	 else
-	 {
+	 else {
 	    ctx->NormalTransform = 0;
 	 }
-      } else {
+      }
+      else {
 	 ctx->NormalTransform = 0;
       }
    }
@@ -1285,4 +1446,3 @@ void gl_update_normal_transform( GLcontext *ctx )
    if (last != ctx->NormalTransform || new_flag)
       ctx->NewState |= NEW_NORMAL_TRANSFORM;
 }
-
