@@ -1,8 +1,8 @@
-/* $Id: stages.c,v 1.13.2.4 2000/03/21 01:26:57 keithw Exp $ */
+/* $Id: stages.c,v 1.20 2000/07/13 18:03:47 keithw Exp $ */
 
 /*
  * Mesa 3-D graphics library
- * Version:  3.1
+ * Version:  3.3
  *
  * Copyright (C) 1999  Brian Paul   All Rights Reserved.
  *
@@ -25,14 +25,10 @@
  */
 
 
-#ifdef HAVE_CONFIG_H
-#include "conf.h"
-#endif
-#ifndef XFree86Server
-#include <stdio.h>
-#else 
-#include "GL/xf86glx.h"
-#endif
+#ifdef PC_HEADER
+#include "all.h"
+#else
+#include "glheader.h"
 #include "bbox.h"
 #include "clip.h"
 #include "context.h"
@@ -43,13 +39,14 @@
 #include "pipeline.h"
 #include "shade.h"
 #include "stages.h"
-#include "types.h"
 #include "translate.h"
+#include "types.h"
 #include "vbcull.h"
 #include "vbindirect.h"
 #include "vbrender.h"
 #include "vbxform.h"
 #include "xform.h"
+#endif
 
 
 static GLmatrix gl_identity_mat;
@@ -189,11 +186,18 @@ static void clean_texcoord( struct vertex_buffer *VB, GLuint i )
    tc->stride = 4 * sizeof(GLfloat);
 }
 
+static void clean_unprojected( struct vertex_buffer *VB )
+{
+   (void) Transform( &VB->Eye,
+                     &gl_identity_mat,
+                     VB->Unprojected,
+                     0,
+                     0);
+   VB->Unprojected = &VB->Eye;
+}
 
 static void clean_clip( struct vertex_buffer *VB )
 {
-/*     printf("clean clip, stride %d, not writable\n", VB->ClipPtr->stride); */
-
    (void) Transform( &VB->Clip,
                      &gl_identity_mat,
                      VB->ClipPtr,
@@ -333,7 +337,11 @@ static void do_vertex_pipeline( struct vertex_buffer *VB )
    }
 
    if (VB->ClipAndMask) {
+      if (MESA_VERBOSE&VERBOSE_CULL)
+	 fprintf(stderr, "Culled in clip\n");
+
       VB->Culled = 1;
+      gl_dont_cull_vb( VB );
       gl_update_materials(VB);
       return;
    }
@@ -348,6 +356,7 @@ static void do_vertex_pipeline( struct vertex_buffer *VB )
 	 if (MESA_VERBOSE&VERBOSE_CULL)
 	    fprintf(stderr, "Culled in userclip\n");
 
+	 gl_dont_cull_vb( VB );
 	 gl_update_materials(VB);
 	 return;
       }
@@ -440,12 +449,23 @@ static void do_normal_transform( struct vertex_buffer *VB )
 
 static void do_lighting( struct vertex_buffer *VB )
 {
-   GLubyte flags = (GLubyte) (VB->CullMode & (CULL_MASK_ACTIVE|COMPACTED_NORMALS));
+   GLubyte flags = (GLubyte) (VB->CullMode & 
+			      (CULL_MASK_ACTIVE|COMPACTED_NORMALS));
 
    if ((flags&CULL_MASK_ACTIVE) && !VB->NormCullStart)
       gl_make_normal_cullmask( VB );
 
-   gl_shade_func_tab[VB->ctx->shade_func_flags | flags]( VB );
+   /* Make sure we can talk about elements 0..2 in the vector we are
+    * lighting.
+    */
+   if (VB->Unprojected->size == 2) {
+      if (VB->Unprojected->flags & VEC_WRITABLE)
+	 gl_vector4f_clean_elem(VB->Unprojected, VB->Count, 2);
+      else 
+	 clean_unprojected( VB );
+   }
+
+   VB->ctx->shade_func_tab[flags]( VB );
 }
 
 
@@ -749,7 +769,7 @@ CONST struct gl_pipeline_stage gl_default_pipeline[] = {
      0, 0,
      DYN_STATE,
      check_fog,
-     gl_fog_vertices },
+     _mesa_fog_vertices },
 
    { "texture gen/transform 0",
      PIPE_OP_TEX0,
