@@ -1,10 +1,10 @@
-/* $Id: vbrender.c,v 1.10.2.7 2000/03/21 01:26:57 keithw Exp $ */
+/* $Id: vbrender.c,v 1.20 2000/07/10 13:22:01 keithw Exp $ */
 
 /*
  * Mesa 3-D graphics library
- * Version:  3.1
+ * Version:  3.3
  * 
- * Copyright (C) 1999  Brian Paul   All Rights Reserved.
+ * Copyright (C) 1999-2000  Brian Paul   All Rights Reserved.
  * 
  * Permission is hereby granted, free of charge, to any person obtaining a
  * copy of this software and associated documentation files (the "Software"),
@@ -41,11 +41,7 @@
 #ifdef PC_HEADER
 #include "all.h"
 #else
-#ifndef XFree86Server
-#include <stdio.h>
-#else
-#include "GL/xf86glx.h"
-#endif
+#include "glheader.h"
 #include "clip.h"
 #include "context.h"
 #include "light.h"
@@ -105,20 +101,14 @@ static INLINE void gl_render_clipped_line2( GLcontext *ctx,
 static void offset_polygon( GLcontext *ctx, GLfloat a, GLfloat b, GLfloat c )
 {
    GLfloat ac, bc, m;
-   GLfloat offset;
+   GLfloat offset = 0.0F;
 
-   if (c<0.001F && c>-0.001F) {
-      /* to prevent underflow problems */
-      offset = 0.0F;
-   }
-   else {
+   if (c*c > 1e-16) {
       ac = a / c;
       bc = b / c;
       if (ac<0.0F)  ac = -ac;
       if (bc<0.0F)  bc = -bc;
       m = MAX2( ac, bc );
-      /* m = sqrt( ac*ac + bc*bc ); */
-
       offset = m * ctx->Polygon.OffsetFactor + ctx->Polygon.OffsetUnits;
    }
 
@@ -130,10 +120,17 @@ static void offset_polygon( GLcontext *ctx, GLfloat a, GLfloat b, GLfloat c )
 #define FLUSH_PRIM(prim)			\
 do {						\
    if (ctx->PB->primitive != prim) {		\
-      gl_reduced_prim_change( ctx, prim );		\
+      gl_reduced_prim_change( ctx, prim );	\
    }						\
 } while(0)
 
+#define FLUSH_POLY(prim)					\
+do {								\
+   if ((ctx->IndirectTriangles & DD_TRI_UNFILLED) == 0) {	\
+      FLUSH_PRIM(prim);						\
+   }								\
+} while(0)
+
 
 /*
  * When glPolygonMode() is used to specify that the front/back rendering
@@ -296,7 +293,7 @@ static void render_triangle( GLcontext *ctx,
       return;
 
    facing = (c<0.0F) ^ (ctx->Polygon.FrontFace==GL_CW);
-   tricaps = ctx->IndirectTriangles;
+   tricaps = ctx->TriangleCaps;
    
    if (tricaps & DD_TRI_OFFSET) {
       /* finish computing plane equation of polygon, compute offset */
@@ -323,6 +320,13 @@ static void render_triangle( GLcontext *ctx,
    else {
       (*ctx->Driver.TriangleFunc)( ctx, v0, v1, v2, pv );
    }
+
+   if (tricaps & DD_TRI_OFFSET) {
+      /* reset Z offsets now */
+      ctx->PointZoffset   = 0.0;
+      ctx->LineZoffset    = 0.0;
+      ctx->PolygonZoffset = 0.0;
+   }
 }
 
 
@@ -346,7 +350,7 @@ static void render_quad( GLcontext *ctx, GLuint v0, GLuint v1,
    GLfloat fy = win[v3][1] - win[v1][1];
    GLfloat c = ex*fy-ey*fx;
    GLuint facing;
-   GLuint tricaps = ctx->IndirectTriangles;
+   GLuint tricaps = ctx->TriangleCaps;
 
    if (c * ctx->backface_sign > 0)
       return;
@@ -381,6 +385,13 @@ static void render_quad( GLcontext *ctx, GLuint v0, GLuint v1,
    else {
       (*ctx->Driver.QuadFunc)( ctx, v0, v1, v2, v3, pv );
    }
+
+   if (tricaps & DD_TRI_OFFSET) {
+      /* reset Z offsets now */
+      ctx->PointZoffset   = 0.0;
+      ctx->LineZoffset    = 0.0;
+      ctx->PolygonZoffset = 0.0;
+   }
 }
 
 
@@ -419,15 +430,16 @@ do {							\
 
 #define EDGEFLAG_POLY_TRI_PRE( i2, i1, i, pv)	\
 do {						\
+  eflag[i2] |= (eflag[i2] >> 2) & 1; 		\
   eflag[i1] |= (eflag[i1] >> 2) & 1; 		\
   eflag[i] |= (eflag[i] >> 2) & 2;		\
 } while (0)
 
 #define EDGEFLAG_POLY_TRI_POST( i2, i1, i, pv)	\
 do {						\
-  eflag[i2] = 0; 		\
+  eflag[i2] &= ~(4|1); 		\
   eflag[i1] &= ~(4|1); 		\
-  eflag[i] &= ~(8|2);		\
+  eflag[i] &= ~(8|2); 		\
 } while (0)
 
 
@@ -507,7 +519,7 @@ do {							\
     (void) vlist; (void) eflag; (void) stipplecounter;
 
 #define TAG(x) x##_cull
-#define INIT(x)  FLUSH_PRIM(x)
+#define INIT(x)  if (x == GL_POLYGON) FLUSH_POLY(x); else FLUSH_PRIM(x)
 #define RESET_STIPPLE *stipplecounter = 0
 #include "render_tmp.h"
 
@@ -545,7 +557,7 @@ do {						\
     GLuint *stipplecounter = &VB->ctx->StippleCounter; \
     (void) eflag; (void) stipplecounter;
 
-#define INIT(x)  FLUSH_PRIM(x);
+#define INIT(x)  if (x == GL_POLYGON) FLUSH_POLY(x); else FLUSH_PRIM(x)
 #define RESET_STIPPLE *stipplecounter = 0
 #include "render_tmp.h"
 
@@ -579,7 +591,7 @@ do {						\
     GLuint *stipplecounter = &VB->ctx->StippleCounter; \
     (void) eflag; (void) stipplecounter;
 
-#define INIT(x)  FLUSH_PRIM(x);
+#define INIT(x)  if (x == GL_POLYGON) FLUSH_POLY(x); else FLUSH_PRIM(x)
 #define TAG(x) x##_clipped
 #define RESET_STIPPLE *stipplecounter = 0
 
@@ -637,9 +649,9 @@ setup_edgeflag( struct vertex_buffer *VB,
    case GL_POLYGON:
       if (flag[0]) flag[0] = 0x1;
       for (i = 1 ; i < n-1 ; i++) {
-	 if (flag[i]) flag[i] = 0x1<<2;
+	 if (flag[i]) flag[i] = 0x4;
       }
-      if (flag[i]) flag[i] = 0x3<<2;
+      if (flag[i]) flag[i] = 0x8;
       break;
    default:
       break;
@@ -723,9 +735,6 @@ void gl_reduced_prim_change( GLcontext *ctx, GLenum prim )
    if (ctx->PB->count > 0) 
       gl_flush_pb(ctx);	
 
-   ctx->PB->count = 0;				
-   ctx->PB->mono = GL_FALSE;			
-   
    if (ctx->PB->primitive != prim) {
       ctx->PB->primitive = prim;			
 
