@@ -1,4 +1,4 @@
-/* $Id: xform_tmp.h,v 1.2 1999/11/05 06:43:11 brianp Exp $ */
+/* $Id: xform_tmp.h,v 1.4 2000/06/27 22:10:00 brianp Exp $ */
 
 /*
  * Mesa 3-D graphics library
@@ -65,8 +65,9 @@
  *     cliped and/or culled vertices.
  */
 
-static void TAG(transform_points1_general)( GLvector4f *to_vec,
-					    const GLmatrix *mat,
+static void _XFORMAPI
+TAG(transform_points1_general)( GLvector4f *to_vec,
+				const GLfloat m[16],
 					    const GLvector4f *from_vec,
 					    const GLubyte *mask,
 					    const GLubyte flag )
@@ -75,7 +76,6 @@ static void TAG(transform_points1_general)( GLvector4f *to_vec,
    GLfloat *from = from_vec->start;
    GLfloat (*to)[4] = (GLfloat (*)[4])to_vec->start;
    GLuint count = from_vec->count;
-   const GLfloat *m = mat->m;
    const GLfloat m0 = m[0],  m12 = m[12];
    const GLfloat m1 = m[1],  m13 = m[13];
    const GLfloat m2 = m[2],  m14 = m[14];
@@ -98,8 +98,9 @@ static void TAG(transform_points1_general)( GLvector4f *to_vec,
    to_vec->count = from_vec->count;
 }
 
-static void TAG(transform_points1_identity)( GLvector4f *to_vec,
-					     const GLmatrix *mat,
+static void _XFORMAPI
+TAG(transform_points1_identity)( GLvector4f *to_vec,
+				 const GLfloat m[16],
 					     const GLvector4f *from_vec,
 					     const GLubyte *mask,
 					     const GLubyte flag )
@@ -109,10 +110,8 @@ static void TAG(transform_points1_identity)( GLvector4f *to_vec,
    GLuint count = from_vec->count;
    GLfloat (*to)[4] = (GLfloat (*)[4])to_vec->start;
    GLuint i;
-   (void) mat;
    (void) mask;
    (void) flag;
-   ASSERT(mat->type == MATRIX_IDENTITY);
    if (to_vec == from_vec) return;
    STRIDE_LOOP {
       CLIP_CHECK {
@@ -125,8 +124,9 @@ static void TAG(transform_points1_identity)( GLvector4f *to_vec,
    to_vec->count = from_vec->count;
 }
 
-static void TAG(transform_points1_2d)( GLvector4f *to_vec,
-				       const GLmatrix *mat,
+static void _XFORMAPI
+TAG(transform_points1_2d)( GLvector4f *to_vec,
+			   const GLfloat m[16],
 				       const GLvector4f *from_vec,
 				       const GLubyte *mask,
 				       const GLubyte flag )
@@ -135,13 +135,11 @@ static void TAG(transform_points1_2d)( GLvector4f *to_vec,
    GLfloat *from = from_vec->start;
    GLfloat (*to)[4] = (GLfloat (*)[4])to_vec->start;
    GLuint count = from_vec->count;
-   const GLfloat *m = mat->m;
    const GLfloat m0 = m[0], m1 = m[1];
    const GLfloat m12 = m[12], m13 = m[13];
    GLuint i;
    (void) mask;
    (void) flag;
-   ASSERT(mat->type == MATRIX_2D);
    STRIDE_LOOP {
       CLIP_CHECK {
 	 const GLfloat ox = from[0];
@@ -154,8 +152,9 @@ static void TAG(transform_points1_2d)( GLvector4f *to_vec,
    to_vec->count = from_vec->count;
 }
 
-static void TAG(transform_points1_2d_no_rot)( GLvector4f *to_vec,
-					      const GLmatrix *mat,
+static void _XFORMAPI
+TAG(transform_points1_2d_no_rot)( GLvector4f *to_vec,
+				  const GLfloat m[16],
 					      const GLvector4f *from_vec,
 					      const GLubyte *mask,
 					      const GLubyte flag )
@@ -164,12 +163,10 @@ static void TAG(transform_points1_2d_no_rot)( GLvector4f *to_vec,
    GLfloat *from = from_vec->start;
    GLfloat (*to)[4] = (GLfloat (*)[4])to_vec->start;
    GLuint count = from_vec->count;
-   const GLfloat *m = mat->m;
    const GLfloat m0 = m[0], m12 = m[12], m13 = m[13];
    GLuint i;
    (void) mask;
    (void) flag;
-   ASSERT(mat->type == MATRIX_2D_NO_ROT);
    STRIDE_LOOP {
       CLIP_CHECK {
 	 const GLfloat ox = from[0];
@@ -183,8 +180,9 @@ static void TAG(transform_points1_2d_no_rot)( GLvector4f *to_vec,
    to_vec->count = from_vec->count;
 }
 
-static void TAG(transform_points1_3d)( GLvector4f *to_vec,
-				       const GLmatrix *mat,
+static void _XFORMAPI
+TAG(transform_points1_3d)( GLvector4f *to_vec,
+			   const GLfloat m[16],
 				       const GLvector4f *from_vec,
 				       const GLubyte *mask,
 				       const GLubyte flag )
@@ -193,13 +191,11 @@ static void TAG(transform_points1_3d)( GLvector4f *to_vec,
    GLfloat *from = from_vec->start;
    GLfloat (*to)[4] = (GLfloat (*)[4])to_vec->start;
    GLuint count = from_vec->count;
-   const GLfloat *m = mat->m;
    const GLfloat m0 = m[0], m1 = m[1], m2 = m[2];
    const GLfloat m12 = m[12], m13 = m[13], m14 = m[14];
    GLuint i;
    (void) mask;
    (void) flag;
-   ASSERT(mat->type == MATRIX_3D);
    STRIDE_LOOP {
       CLIP_CHECK {
 	 const GLfloat ox = from[0];
@@ -214,8 +210,9 @@ static void TAG(transform_points1_3d)( GLvector4f *to_vec,
 }
 
 
-static void TAG(transform_points1_3d_no_rot)( GLvector4f *to_vec,
-					      const GLmatrix *mat,
+static void _XFORMAPI
+TAG(transform_points1_3d_no_rot)( GLvector4f *to_vec,
+				  const GLfloat m[16],
 					      const GLvector4f *from_vec,
 					      const GLubyte *mask,
 					      const GLubyte flag )
@@ -224,13 +221,11 @@ static void TAG(transform_points1_3d_no_rot)( GLvector4f *to_vec,
    GLfloat *from = from_vec->start;
    GLfloat (*to)[4] = (GLfloat (*)[4])to_vec->start;
    GLuint count = from_vec->count;
-   const GLfloat *m = mat->m;
    const GLfloat m0 = m[0];
    const GLfloat m12 = m[12], m13 = m[13], m14 = m[14];
    GLuint i;
    (void) mask;
    (void) flag;
-   ASSERT(mat->type == MATRIX_3D_NO_ROT);
    STRIDE_LOOP {
       CLIP_CHECK {
 	 const GLfloat ox = from[0];
@@ -244,8 +239,9 @@ static void TAG(transform_points1_3d_no_rot)( GLvector4f *to_vec,
    to_vec->count = from_vec->count;
 }
 
-static void TAG(transform_points1_perspective)( GLvector4f *to_vec,
-						const GLmatrix *mat,
+static void _XFORMAPI
+TAG(transform_points1_perspective)( GLvector4f *to_vec,
+				    const GLfloat m[16],
 						const GLvector4f *from_vec,
 						const GLubyte *mask,
 						const GLubyte flag )
@@ -254,12 +250,10 @@ static void TAG(transform_points1_perspective)( GLvector4f *to_vec,
    GLfloat *from = from_vec->start;
    GLfloat (*to)[4] = (GLfloat (*)[4])to_vec->start;
    GLuint count = from_vec->count;
-   const GLfloat *m = mat->m;
    const GLfloat m0 = m[0], m14 = m[14];
    GLuint i;
    (void) mask;
    (void) flag;
-   ASSERT(mat->type == MATRIX_PERSPECTIVE);
    STRIDE_LOOP {
       CLIP_CHECK {
 	 const GLfloat ox = from[0];
@@ -281,8 +275,9 @@ static void TAG(transform_points1_perspective)( GLvector4f *to_vec,
  * present early in the geometry pipeline and throughout the
  * texture pipeline.
  */
-static void TAG(transform_points2_general)( GLvector4f *to_vec,
-					    const GLmatrix *mat,
+static void _XFORMAPI
+TAG(transform_points2_general)( GLvector4f *to_vec,
+				const GLfloat m[16],
 					    const GLvector4f *from_vec,
 					    const GLubyte *mask,
 					    const GLubyte flag )
@@ -291,7 +286,6 @@ static void TAG(transform_points2_general)( GLvector4f *to_vec,
    GLfloat *from = from_vec->start;
    GLfloat (*to)[4] = (GLfloat (*)[4])to_vec->start;
    GLuint count = from_vec->count;
-   const GLfloat *m = mat->m;
    const GLfloat m0 = m[0],  m4 = m[4],  m12 = m[12];
    const GLfloat m1 = m[1],  m5 = m[5],  m13 = m[13];
    const GLfloat m2 = m[2],  m6 = m[6],  m14 = m[14];
@@ -313,8 +307,9 @@ static void TAG(transform_points2_general)( GLvector4f *to_vec,
    to_vec->count = from_vec->count;
 }
 
-static void TAG(transform_points2_identity)( GLvector4f *to_vec,
-					     const GLmatrix *mat,
+static void _XFORMAPI
+TAG(transform_points2_identity)( GLvector4f *to_vec,
+				 const GLfloat m[16],
 					     const GLvector4f *from_vec,
 					     const GLubyte *mask,
 					     const GLubyte flag )
@@ -324,10 +319,8 @@ static void TAG(transform_points2_identity)( GLvector4f *to_vec,
    GLfloat (*to)[4] = (GLfloat (*)[4])to_vec->start;
    GLuint count = from_vec->count;
    GLuint i;
-   (void) mat;
    (void) mask;
    (void) flag;
-   ASSERT(mat->type == MATRIX_IDENTITY);
    if (to_vec == from_vec) return;
    STRIDE_LOOP {
       CLIP_CHECK {
@@ -340,8 +333,9 @@ static void TAG(transform_points2_identity)( GLvector4f *to_vec,
    to_vec->count = from_vec->count;
 }
 
-static void TAG(transform_points2_2d)( GLvector4f *to_vec,
-				       const GLmatrix *mat,
+static void _XFORMAPI
+TAG(transform_points2_2d)( GLvector4f *to_vec,
+			   const GLfloat m[16],
 				       const GLvector4f *from_vec,
 				       const GLubyte *mask,
 				       const GLubyte flag )
@@ -350,13 +344,11 @@ static void TAG(transform_points2_2d)( GLvector4f *to_vec,
    GLfloat *from = from_vec->start;
    GLfloat (*to)[4] = (GLfloat (*)[4])to_vec->start;
    GLuint count = from_vec->count;
-   const GLfloat *m = mat->m;
    const GLfloat m0 = m[0], m1 = m[1], m4 = m[4], m5 = m[5];
    const GLfloat m12 = m[12], m13 = m[13];
    GLuint i;
    (void) mask;
    (void) flag;
-   ASSERT(mat->type == MATRIX_2D);
    STRIDE_LOOP {
       CLIP_CHECK {
 	 const GLfloat ox = from[0], oy = from[1];
@@ -370,8 +362,9 @@ static void TAG(transform_points2_2d)( GLvector4f *to_vec,
    to_vec->count = from_vec->count;
 }
 
-static void TAG(transform_points2_2d_no_rot)( GLvector4f *to_vec,
-					      const GLmatrix *mat,
+static void _XFORMAPI
+TAG(transform_points2_2d_no_rot)( GLvector4f *to_vec,
+				  const GLfloat m[16],
 					      const GLvector4f *from_vec,
 					      const GLubyte *mask,
 					      const GLubyte flag )
@@ -380,12 +373,10 @@ static void TAG(transform_points2_2d_no_rot)( GLvector4f *to_vec,
    GLfloat *from = from_vec->start;
    GLfloat (*to)[4] = (GLfloat (*)[4])to_vec->start;
    GLuint count = from_vec->count;
-   const GLfloat *m = mat->m;
    const GLfloat m0 = m[0], m5 = m[5], m12 = m[12], m13 = m[13];
    GLuint i;
    (void) mask;
    (void) flag;
-   ASSERT(mat->type == MATRIX_2D_NO_ROT);
    STRIDE_LOOP {
       CLIP_CHECK {
 	 const GLfloat ox = from[0], oy = from[1];
@@ -399,8 +390,9 @@ static void TAG(transform_points2_2d_no_rot)( GLvector4f *to_vec,
    to_vec->count = from_vec->count;
 }
 
-static void TAG(transform_points2_3d)( GLvector4f *to_vec,
-				       const GLmatrix *mat,
+static void _XFORMAPI
+TAG(transform_points2_3d)( GLvector4f *to_vec,
+			   const GLfloat m[16],
 				       const GLvector4f *from_vec,
 				       const GLubyte *mask,
 				       const GLubyte flag )
@@ -409,13 +401,11 @@ static void TAG(transform_points2_3d)( GLvector4f *to_vec,
    GLfloat *from = from_vec->start;
    GLfloat (*to)[4] = (GLfloat (*)[4])to_vec->start;
    GLuint count = from_vec->count;
-   const GLfloat *m = mat->m;
    const GLfloat m0 = m[0], m1 = m[1], m2 = m[2], m4 = m[4], m5 = m[5];
    const GLfloat m6 = m[6], m12 = m[12], m13 = m[13], m14 = m[14];
    GLuint i;
    (void) mask;
    (void) flag;
-   ASSERT(mat->type == MATRIX_3D);
    STRIDE_LOOP {
       CLIP_CHECK {
 	 const GLfloat ox = from[0], oy = from[1];
@@ -433,8 +423,9 @@ static void TAG(transform_points2_3d)( GLvector4f *to_vec,
 /* I would actually say this was a fairly important function, from
  * a texture transformation point of view.
  */
-static void TAG(transform_points2_3d_no_rot)( GLvector4f *to_vec,
-					      const GLmatrix *mat,
+static void _XFORMAPI
+TAG(transform_points2_3d_no_rot)( GLvector4f *to_vec,
+				  const GLfloat m[16],
 					      const GLvector4f *from_vec,
 					      const GLubyte *mask,
 					      const GLubyte flag )
@@ -443,13 +434,11 @@ static void TAG(transform_points2_3d_no_rot)( GLvector4f *to_vec,
    GLfloat *from = from_vec->start;
    GLfloat (*to)[4] = (GLfloat (*)[4])to_vec->start;
    GLuint count = from_vec->count;
-   const GLfloat *m = mat->m;
    const GLfloat m0 = m[0], m5 = m[5];
    const GLfloat m12 = m[12], m13 = m[13], m14 = m[14];
    GLuint i;
    (void) mask;
    (void) flag;
-   ASSERT(mat->type == MATRIX_3D_NO_ROT);
    STRIDE_LOOP {
       CLIP_CHECK {
 	 const GLfloat ox = from[0], oy = from[1];
@@ -472,8 +461,9 @@ static void TAG(transform_points2_3d_no_rot)( GLvector4f *to_vec,
  * code.  It's also hard to remove any of these functions if you are
  * attached to the assertions that have appeared in them.
  */
-static void TAG(transform_points2_perspective)( GLvector4f *to_vec,
-						const GLmatrix *mat,
+static void _XFORMAPI
+TAG(transform_points2_perspective)( GLvector4f *to_vec,
+				    const GLfloat m[16],
 						const GLvector4f *from_vec,
 						const GLubyte *mask,
 						const GLubyte flag )
@@ -482,12 +472,10 @@ static void TAG(transform_points2_perspective)( GLvector4f *to_vec,
    GLfloat *from = from_vec->start;
    GLfloat (*to)[4] = (GLfloat (*)[4])to_vec->start;
    GLuint count = from_vec->count;
-   const GLfloat *m = mat->m;
    const GLfloat m0 = m[0], m5 = m[5], m14 = m[14];
    GLuint i;
    (void) mask;
    (void) flag;
-   ASSERT(mat->type == MATRIX_PERSPECTIVE);
    STRIDE_LOOP {
       CLIP_CHECK {
 	 const GLfloat ox = from[0], oy = from[1];
@@ -504,8 +492,9 @@ static void TAG(transform_points2_perspective)( GLvector4f *to_vec,
 
 
 
-static void TAG(transform_points3_general)( GLvector4f *to_vec,
-					    const GLmatrix *mat,
+static void _XFORMAPI
+TAG(transform_points3_general)( GLvector4f *to_vec,
+				const GLfloat m[16],
 					    const GLvector4f *from_vec,
 					    const GLubyte *mask,
 					    const GLubyte flag )
@@ -514,7 +503,6 @@ static void TAG(transform_points3_general)( GLvector4f *to_vec,
    GLfloat *from = from_vec->start;
    GLfloat (*to)[4] = (GLfloat (*)[4])to_vec->start;
    GLuint count = from_vec->count;
-   const GLfloat *m = mat->m;
    const GLfloat m0 = m[0],  m4 = m[4],  m8 = m[8],  m12 = m[12];
    const GLfloat m1 = m[1],  m5 = m[5],  m9 = m[9],  m13 = m[13];
    const GLfloat m2 = m[2],  m6 = m[6],  m10 = m[10],  m14 = m[14];
@@ -522,7 +510,6 @@ static void TAG(transform_points3_general)( GLvector4f *to_vec,
    GLuint i;
    (void) mask;
    (void) flag;
-   ASSERT(mat->type == MATRIX_GENERAL);
    STRIDE_LOOP {
       CLIP_CHECK {
 	 const GLfloat ox = from[0], oy = from[1], oz = from[2];
@@ -537,8 +524,9 @@ static void TAG(transform_points3_general)( GLvector4f *to_vec,
    to_vec->count = from_vec->count;
 }
 
-static void TAG(transform_points3_identity)( GLvector4f *to_vec,
-					     const GLmatrix *mat,
+static void _XFORMAPI
+TAG(transform_points3_identity)( GLvector4f *to_vec,
+				 const GLfloat m[16],
 					     const GLvector4f *from_vec,
 					     const GLubyte *mask,
 					     const GLubyte flag )
@@ -548,10 +536,8 @@ static void TAG(transform_points3_identity)( GLvector4f *to_vec,
    GLfloat (*to)[4] = (GLfloat (*)[4])to_vec->start;
    GLuint count = from_vec->count;
    GLuint i;
-   (void) mat;
    (void) mask;
    (void) flag;
-   ASSERT(mat->type == MATRIX_IDENTITY);
    if (to_vec == from_vec) return;
    STRIDE_LOOP {
       CLIP_CHECK {
@@ -565,8 +551,9 @@ static void TAG(transform_points3_identity)( GLvector4f *to_vec,
    to_vec->count = from_vec->count;
 }
 
-static void TAG(transform_points3_2d)( GLvector4f *to_vec,
-				       const GLmatrix *mat,
+static void _XFORMAPI
+TAG(transform_points3_2d)( GLvector4f *to_vec,
+			   const GLfloat m[16],
 				       const GLvector4f *from_vec,
 				       const GLubyte *mask,
 				       const GLubyte flag )
@@ -575,13 +562,11 @@ static void TAG(transform_points3_2d)( GLvector4f *to_vec,
    GLfloat *from = from_vec->start;
    GLfloat (*to)[4] = (GLfloat (*)[4])to_vec->start;
    GLuint count = from_vec->count;
-   const GLfloat *m = mat->m;
    const GLfloat m0 = m[0], m1 = m[1], m4 = m[4], m5 = m[5];
    const GLfloat m12 = m[12], m13 = m[13];
    GLuint i;
    (void) mask;
    (void) flag;
-   ASSERT(mat->type == MATRIX_2D);
    STRIDE_LOOP {
       CLIP_CHECK {
 	 const GLfloat ox = from[0], oy = from[1], oz = from[2];
@@ -595,8 +580,9 @@ static void TAG(transform_points3_2d)( GLvector4f *to_vec,
    to_vec->count = from_vec->count;
 }
 
-static void TAG(transform_points3_2d_no_rot)( GLvector4f *to_vec,
-					      const GLmatrix *mat,
+static void _XFORMAPI
+TAG(transform_points3_2d_no_rot)( GLvector4f *to_vec,
+				  const GLfloat m[16],
 					      const GLvector4f *from_vec,
 					      const GLubyte *mask,
 					      const GLubyte flag )
@@ -605,12 +591,10 @@ static void TAG(transform_points3_2d_no_rot)( GLvector4f *to_vec,
    GLfloat *from = from_vec->start;
    GLfloat (*to)[4] = (GLfloat (*)[4])to_vec->start;
    GLuint count = from_vec->count;
-   const GLfloat *m = mat->m;
    const GLfloat m0 = m[0], m5 = m[5], m12 = m[12], m13 = m[13];
    GLuint i;
    (void) mask;
    (void) flag;
-   ASSERT(mat->type == MATRIX_2D_NO_ROT);
    STRIDE_LOOP {
       CLIP_CHECK {
 	 const GLfloat ox = from[0], oy = from[1], oz = from[2];
@@ -624,8 +608,9 @@ static void TAG(transform_points3_2d_no_rot)( GLvector4f *to_vec,
    to_vec->count = from_vec->count;
 }
 
-static void TAG(transform_points3_3d)( GLvector4f *to_vec,
-				       const GLmatrix *mat,
+static void _XFORMAPI
+TAG(transform_points3_3d)( GLvector4f *to_vec,
+			   const GLfloat m[16],
 				       const GLvector4f *from_vec,
 				       const GLubyte *mask,
 				       const GLubyte flag )
@@ -634,14 +619,12 @@ static void TAG(transform_points3_3d)( GLvector4f *to_vec,
    GLfloat *from = from_vec->start;
    GLfloat (*to)[4] = (GLfloat (*)[4])to_vec->start;
    GLuint count = from_vec->count;
-   const GLfloat *m = mat->m;
    const GLfloat m0 = m[0], m1 = m[1], m2 = m[2], m4 = m[4], m5 = m[5];
    const GLfloat m6 = m[6], m8 = m[8], m9 = m[9], m10 = m[10];
    const GLfloat m12 = m[12], m13 = m[13], m14 = m[14];
    GLuint i;
    (void) mask;
    (void) flag;
-   ASSERT(mat->type == MATRIX_3D);
    STRIDE_LOOP {
       CLIP_CHECK {
 	 const GLfloat ox = from[0], oy = from[1], oz = from[2];
@@ -657,8 +640,9 @@ static void TAG(transform_points3_3d)( GLvector4f *to_vec,
 
 /* previously known as ortho...
  */
-static void TAG(transform_points3_3d_no_rot)( GLvector4f *to_vec,
-					      const GLmatrix *mat,
+static void _XFORMAPI
+TAG(transform_points3_3d_no_rot)( GLvector4f *to_vec,
+				  const GLfloat m[16],
 					      const GLvector4f *from_vec,
 					      const GLubyte *mask,
 					      const GLubyte flag )
@@ -667,13 +651,11 @@ static void TAG(transform_points3_3d_no_rot)( GLvector4f *to_vec,
    GLfloat *from = from_vec->start;
    GLfloat (*to)[4] = (GLfloat (*)[4])to_vec->start;
    GLuint count = from_vec->count;
-   const GLfloat *m = mat->m;
    const GLfloat m0 = m[0], m5 = m[5];
    const GLfloat m10 = m[10], m12 = m[12], m13 = m[13], m14 = m[14];
    GLuint i;
    (void) mask;
    (void) flag;
-   ASSERT(mat->type == MATRIX_3D_NO_ROT);
    STRIDE_LOOP {
       CLIP_CHECK {
 	 const GLfloat ox = from[0], oy = from[1], oz = from[2];
@@ -687,8 +669,9 @@ static void TAG(transform_points3_3d_no_rot)( GLvector4f *to_vec,
    to_vec->count = from_vec->count;
 }
 
-static void TAG(transform_points3_perspective)( GLvector4f *to_vec,
-						const GLmatrix *mat,
+static void _XFORMAPI
+TAG(transform_points3_perspective)( GLvector4f *to_vec,
+				    const GLfloat m[16],
 						const GLvector4f *from_vec,
 						const GLubyte *mask,
 						const GLubyte flag )
@@ -697,13 +680,11 @@ static void TAG(transform_points3_perspective)( GLvector4f *to_vec,
    GLfloat *from = from_vec->start;
    GLfloat (*to)[4] = (GLfloat (*)[4])to_vec->start;
    GLuint count = from_vec->count;
-   const GLfloat *m = mat->m;
    const GLfloat m0 = m[0], m5 = m[5], m8 = m[8], m9 = m[9];
    const GLfloat m10 = m[10], m14 = m[14];
    GLuint i;
    (void) mask;
    (void) flag;
-   ASSERT(mat->type == MATRIX_PERSPECTIVE);
    STRIDE_LOOP {
       CLIP_CHECK {
 	 const GLfloat ox = from[0], oy = from[1], oz = from[2];
@@ -720,8 +701,9 @@ static void TAG(transform_points3_perspective)( GLvector4f *to_vec,
 
 
 
-static void TAG(transform_points4_general)( GLvector4f *to_vec,
-					    const GLmatrix *mat,
+static void _XFORMAPI
+TAG(transform_points4_general)( GLvector4f *to_vec,
+				const GLfloat m[16],
 					    const GLvector4f *from_vec,
 					    const GLubyte *mask,
 					    const GLubyte flag )
@@ -730,7 +712,6 @@ static void TAG(transform_points4_general)( GLvector4f *to_vec,
    GLfloat *from = from_vec->start;
    GLfloat (*to)[4] = (GLfloat (*)[4])to_vec->start;
    GLuint count = from_vec->count;
-   const GLfloat *m = mat->m;
    const GLfloat m0 = m[0],  m4 = m[4],  m8 = m[8],  m12 = m[12];
    const GLfloat m1 = m[1],  m5 = m[5],  m9 = m[9],  m13 = m[13];
    const GLfloat m2 = m[2],  m6 = m[6],  m10 = m[10],  m14 = m[14];
@@ -738,7 +719,6 @@ static void TAG(transform_points4_general)( GLvector4f *to_vec,
    GLuint i;
    (void) mask;
    (void) flag;
-   ASSERT(mat->type == MATRIX_GENERAL);
    STRIDE_LOOP {
       CLIP_CHECK {
 	 const GLfloat ox = from[0], oy = from[1], oz = from[2], ow = from[3];
@@ -753,8 +733,9 @@ static void TAG(transform_points4_general)( GLvector4f *to_vec,
    to_vec->count = from_vec->count;
 }
 
-static void TAG(transform_points4_identity)( GLvector4f *to_vec,
-					     const GLmatrix *mat,
+static void _XFORMAPI
+TAG(transform_points4_identity)( GLvector4f *to_vec,
+				 const GLfloat m[16],
 					     const GLvector4f *from_vec,
 					     const GLubyte *mask,
 					     const GLubyte flag )
@@ -764,10 +745,8 @@ static void TAG(transform_points4_identity)( GLvector4f *to_vec,
    GLfloat (*to)[4] = (GLfloat (*)[4])to_vec->start;
    GLuint count = from_vec->count;
    GLuint i;
-   (void) mat;
    (void) mask;
    (void) flag;
-   ASSERT(mat->type == MATRIX_IDENTITY);
    if (to_vec == from_vec) return;
    STRIDE_LOOP {
       CLIP_CHECK {
@@ -782,8 +761,9 @@ static void TAG(transform_points4_identity)( GLvector4f *to_vec,
    to_vec->count = from_vec->count;
 }
 
-static void TAG(transform_points4_2d)( GLvector4f *to_vec,
-				       const GLmatrix *mat,
+static void _XFORMAPI
+TAG(transform_points4_2d)( GLvector4f *to_vec,
+			   const GLfloat m[16],
 				       const GLvector4f *from_vec,
 				       const GLubyte *mask,
 				       const GLubyte flag )
@@ -792,13 +772,11 @@ static void TAG(transform_points4_2d)( GLvector4f *to_vec,
    GLfloat *from = from_vec->start;
    GLfloat (*to)[4] = (GLfloat (*)[4])to_vec->start;
    GLuint count = from_vec->count;
-   const GLfloat *m = mat->m;
    const GLfloat m0 = m[0], m1 = m[1], m4 = m[4], m5 = m[5];
    const GLfloat m12 = m[12], m13 = m[13];
    GLuint i;
    (void) mask;
    (void) flag;
-   ASSERT(mat->type == MATRIX_2D);
    STRIDE_LOOP {
       CLIP_CHECK {
 	 const GLfloat ox = from[0], oy = from[1], oz = from[2], ow = from[3];
@@ -813,8 +791,9 @@ static void TAG(transform_points4_2d)( GLvector4f *to_vec,
    to_vec->count = from_vec->count;
 }
 
-static void TAG(transform_points4_2d_no_rot)( GLvector4f *to_vec,
-					      const GLmatrix *mat,
+static void _XFORMAPI
+TAG(transform_points4_2d_no_rot)( GLvector4f *to_vec,
+				  const GLfloat m[16],
 					      const GLvector4f *from_vec,
 					      const GLubyte *mask,
 					      const GLubyte flag )
@@ -823,12 +802,10 @@ static void TAG(transform_points4_2d_no_rot)( GLvector4f *to_vec,
    GLfloat *from = from_vec->start;
    GLfloat (*to)[4] = (GLfloat (*)[4])to_vec->start;
    GLuint count = from_vec->count;
-   const GLfloat *m = mat->m;
    const GLfloat m0 = m[0], m5 = m[5], m12 = m[12], m13 = m[13];
    GLuint i;
    (void) mask;
    (void) flag;
-   ASSERT(mat->type == MATRIX_2D_NO_ROT);
    STRIDE_LOOP {
       CLIP_CHECK {
 	 const GLfloat ox = from[0], oy = from[1], oz = from[2], ow = from[3];
@@ -843,8 +820,9 @@ static void TAG(transform_points4_2d_no_rot)( GLvector4f *to_vec,
    to_vec->count = from_vec->count;
 }
 
-static void TAG(transform_points4_3d)( GLvector4f *to_vec,
-				       const GLmatrix *mat,
+static void _XFORMAPI
+TAG(transform_points4_3d)( GLvector4f *to_vec,
+			   const GLfloat m[16],
 				       const GLvector4f *from_vec,
 				       const GLubyte *mask,
 				       const GLubyte flag )
@@ -853,14 +831,12 @@ static void TAG(transform_points4_3d)( GLvector4f *to_vec,
    GLfloat *from = from_vec->start;
    GLfloat (*to)[4] = (GLfloat (*)[4])to_vec->start;
    GLuint count = from_vec->count;
-   const GLfloat *m = mat->m;
    const GLfloat m0 = m[0], m1 = m[1], m2 = m[2], m4 = m[4], m5 = m[5];
    const GLfloat m6 = m[6], m8 = m[8], m9 = m[9], m10 = m[10];
    const GLfloat m12 = m[12], m13 = m[13], m14 = m[14];
    GLuint i;
    (void) mask;
    (void) flag;
-   ASSERT(mat->type == MATRIX_3D);
    STRIDE_LOOP {
       CLIP_CHECK {
 	 const GLfloat ox = from[0], oy = from[1], oz = from[2], ow = from[3];
@@ -875,8 +851,9 @@ static void TAG(transform_points4_3d)( GLvector4f *to_vec,
    to_vec->count = from_vec->count;
 }
 
-static void TAG(transform_points4_3d_no_rot)( GLvector4f *to_vec,
-					      const GLmatrix *mat,
+static void _XFORMAPI
+TAG(transform_points4_3d_no_rot)( GLvector4f *to_vec,
+				  const GLfloat m[16],
 					      const GLvector4f *from_vec,
 					      const GLubyte *mask,
 					      const GLubyte flag )
@@ -885,13 +862,11 @@ static void TAG(transform_points4_3d_no_rot)( GLvector4f *to_vec,
    GLfloat *from = from_vec->start;
    GLfloat (*to)[4] = (GLfloat (*)[4])to_vec->start;
    GLuint count = from_vec->count;
-   const GLfloat *m = mat->m;
    const GLfloat m0 = m[0], m5 = m[5];
    const GLfloat m10 = m[10], m12 = m[12], m13 = m[13], m14 = m[14];
    GLuint i;
    (void) mask;
    (void) flag;
-   ASSERT(mat->type == MATRIX_3D_NO_ROT);
    STRIDE_LOOP {
       CLIP_CHECK {
 	 const GLfloat ox = from[0], oy = from[1], oz = from[2], ow = from[3];
@@ -906,8 +881,9 @@ static void TAG(transform_points4_3d_no_rot)( GLvector4f *to_vec,
    to_vec->count = from_vec->count;
 }
 
-static void TAG(transform_points4_perspective)( GLvector4f *to_vec,
-						const GLmatrix *mat,
+static void _XFORMAPI
+TAG(transform_points4_perspective)( GLvector4f *to_vec,
+				    const GLfloat m[16],
 						const GLvector4f *from_vec,
 						const GLubyte *mask,
 						const GLubyte flag )
@@ -916,13 +892,11 @@ static void TAG(transform_points4_perspective)( GLvector4f *to_vec,
    GLfloat *from = from_vec->start;
    GLfloat (*to)[4] = (GLfloat (*)[4])to_vec->start;
    GLuint count = from_vec->count;
-   const GLfloat *m = mat->m;
    const GLfloat m0 = m[0], m5 = m[5], m8 = m[8], m9 = m[9];
    const GLfloat m10 = m[10], m14 = m[14];
    GLuint i;
    (void) mask;
    (void) flag;
-   ASSERT(mat->type == MATRIX_PERSPECTIVE);
    STRIDE_LOOP {
       CLIP_CHECK {
 	 const GLfloat ox = from[0], oy = from[1], oz = from[2], ow = from[3];
@@ -938,16 +912,16 @@ static void TAG(transform_points4_perspective)( GLvector4f *to_vec,
    to_vec->count = from_vec->count;
 }
 
-static transform_func TAG(transform_tab_1)[7];
-static transform_func TAG(transform_tab_2)[7];
-static transform_func TAG(transform_tab_3)[7];
-static transform_func TAG(transform_tab_4)[7];
+static transform_func _XFORMAPI TAG(transform_tab_1)[7];
+static transform_func _XFORMAPI TAG(transform_tab_2)[7];
+static transform_func _XFORMAPI TAG(transform_tab_3)[7];
+static transform_func _XFORMAPI TAG(transform_tab_4)[7];
 
 /* Similar functions could be called several times, with more highly
  * optimized routines overwriting the arrays.  This only occurs during
  * startup.
  */
-static void TAG(init_c_transformations)( void )
+static void _XFORMAPI TAG(init_c_transformations)( void )
 {
 #define TAG_TAB   gl_transform_tab[IDX]
 #define TAG_TAB_1 TAG(transform_tab_1)
